import re
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient

new_value = "will_break_next"


def test_cmakedeps_direct_deps_paths():
    c = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan.tools.files import copy
        from conan import ConanFile
        class TestConan(ConanFile):
            name = "lib"
            version = "1.0"
            def package_info(self):
                self.cpp_info.includedirs = ["myincludes"]
                self.cpp_info.libdirs = ["mylib"]
                self.cpp_info.frameworkdirs = ["myframework"]
    """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")
    conanfile = textwrap.dedent(f"""
        from conan import ConanFile
        from conan.tools.cmake import CMake
        class PkgConan(ConanFile):
            requires = "lib/1.0"
            settings = "os", "arch", "compiler", "build_type"
            generators = "CMakeDeps"
    """)
    c.save({"conanfile.py": conanfile}, clean_first=True)
    c.run(f"install . -c tools.cmake.cmakedeps:new={new_value}")
    cmake_paths = c.load("conan_cmakedeps_paths.cmake")
    assert "set(CMAKE_FIND_PACKAGE_PREFER_CONFIG ON)" in cmake_paths
    assert re.search(r"list\(PREPEND CMAKE_PROGRAM_PATH \".*/bin\"", cmake_paths)  # default
    assert re.search(r"list\(PREPEND CMAKE_LIBRARY_PATH \".*/mylib\"", cmake_paths)
    assert re.search(r"list\(PREPEND CMAKE_INCLUDE_PATH \".*/myincludes\"", cmake_paths)
    assert re.search(r"list\(PREPEND CMAKE_FRAMEWORK_PATH \".*/myframework\"", cmake_paths)


def test_cmakedeps_transitive_paths():
    c = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan.tools.files import copy
        from conan import ConanFile
        class TestConan(ConanFile):
            name = "liba"
            version = "1.0"
            def package_info(self):
                self.cpp_info.includedirs = ["includea"]
                self.cpp_info.libdirs = ["liba"]
                self.cpp_info.bindirs = ["bina"]
    """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")
    conanfile = textwrap.dedent("""
        import os
        from conan.tools.files import copy
        from conan import ConanFile
        class TestConan(ConanFile):
            name = "libb"
            version = "1.0"
            requires = "liba/1.0"
            def package_info(self):
                self.cpp_info.includedirs = ["includeb"]
                self.cpp_info.libdirs = ["libb"]
                self.cpp_info.bindirs = ["binb"]
    """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")
    conanfile = textwrap.dedent(f"""
        from conan import ConanFile
        class PkgConan(ConanFile):
            requires = "libb/1.0"
            settings = "os", "arch", "compiler", "build_type"
            generators = "CMakeDeps"
    """)
    c.save({"conanfile.py": conanfile}, clean_first=True)
    c.run(f"install . -c tools.cmake.cmakedeps:new={new_value}")
    cmake_paths = c.load("conan_cmakedeps_paths.cmake")
    assert re.search(r"list\(PREPEND CMAKE_PROGRAM_PATH \".*/libb.*/p/binb\"\)", cmake_paths)
    assert not re.search(r"list\(PREPEND CMAKE_PROGRAM_PATH /bina\"", cmake_paths)
    assert re.search(r"list\(PREPEND CMAKE_LIBRARY_PATH \".*/libb.*/p/libb\" \".*/liba.*/p/liba\"\)",
                     cmake_paths)
    assert re.search(r"list\(PREPEND CMAKE_INCLUDE_PATH \".*/libb.*/p/includeb\" "
                     r"\".*/liba.*/p/includea\"\)", cmake_paths)


def test_cmakedeps_deployer_relative_paths():
    c = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan.tools.files import copy
        from conan import ConanFile
        class TestConan(ConanFile):
            name = "liba"
            version = "1.0"

            def package_info(self):
                self.cpp_info.includedirs = ["includea"]
                self.cpp_info.libdirs = ["bina"]
                self.cpp_info.bindirs = ["bina"]
                crypto_module = os.path.join("share", "cmake", "crypto.cmake")
                self.cpp_info.set_property("cmake_build_modules", [crypto_module])
    """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")

    conanfile_cmake = textwrap.dedent("""
        import os
        from conan.tools.files import save
        from conan import ConanFile
        class TestConan(ConanFile):
            name = "libb"
            version = "1.0"

            def package(self):
                save(self, os.path.join(self.package_folder, "libb-config.cmake"), "")
            def package_info(self):
                self.cpp_info.set_property("cmake_find_mode", "none")
        """)

    c.save({"conanfile.py": conanfile_cmake})
    c.run("create .")
    conanfile = textwrap.dedent(f"""
        from conan import ConanFile
        from conan.tools.cmake import CMake
        class PkgConan(ConanFile):
            requires = "liba/1.0", "libb/1.0"
            settings = "os", "arch", "compiler", "build_type"
            generators = "CMakeDeps"
    """)
    c.save({"conanfile.py": conanfile}, clean_first=True)

    # Now with a deployment
    c.run(f"install . -c tools.cmake.cmakedeps:new={new_value} --deployer=full_deploy")
    cmake_paths = c.load("conan_cmakedeps_paths.cmake")
    assert 'set(libb_DIR "${CMAKE_CURRENT_LIST_DIR}/full_deploy/host/libb/1.0")' in cmake_paths
    assert ('set(CONAN_RUNTIME_LIB_DIRS "$<$<CONFIG:Release>:${CMAKE_CURRENT_LIST_DIR}'
            '/full_deploy/host/liba/1.0/bina>"') in cmake_paths
    liba_config = c.load("liba-config.cmake")
    assert ('include("${CMAKE_CURRENT_LIST_DIR}/full_deploy/'
            'host/liba/1.0/share/cmake/crypto.cmake")') in liba_config
    assert ('set(liba_INCLUDE_DIRS "${CMAKE_CURRENT_LIST_DIR}/full_deploy/'
            'host/liba/1.0/includea" )') in liba_config
    liba_targets = c.load("liba-Targets-release.cmake")
    assert ('set(liba_PACKAGE_FOLDER_RELEASE "${CMAKE_CURRENT_LIST_DIR}/full_deploy/'
            'host/liba/1.0")') in liba_targets


def test_cmakeconfigdeps_recipe():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan.tools.cmake import CMakeConfigDeps
        from conan import ConanFile
        class TestConan(ConanFile):
            settings = "build_type"
            requires = "dep/0.1"
            def generate(self):
                deps = CMakeConfigDeps(self)
                deps.generate()
    """)
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
            "app/conanfile.py": conanfile})
    c.run("create dep")
    c.run("install app", assert_error=True)
    assert "CMakeConfigDeps is being used in conanfile, but the conf " \
           "'tools.cmake.cmakedeps:new' is not enabled" in c.out
    c.run("install app -c tools.cmake.cmakedeps:new=will_break_next")
    # will not fail, still warn
    assert "WARN: Using the new CMakeConfigDeps generator" in c.out
    # The only-recipe also not fails
    c.run("install app -c tools.cmake.cmakedeps:new=recipe_will_break")
    # will not fail
    assert "WARN: Using the new CMakeConfigDeps generator" in c.out

    # attribute generator
    conanfile = textwrap.dedent("""
        from conan.tools.cmake import CMakeConfigDeps
        from conan import ConanFile
        class TestConan(ConanFile):
            settings = "build_type"
            requires = "dep/0.1"
            generators = "CMakeConfigDeps"
        """)
    c.save({"app/conanfile.py": conanfile}, clean_first=True)
    c.run("install app", assert_error=True)
    assert "CMakeConfigDeps is being used in conanfile, but the conf " \
           "'tools.cmake.cmakedeps:new' is not enabled" in c.out
    c.run("install app -c tools.cmake.cmakedeps:new=will_break_next")
    assert "WARN: Using the new CMakeConfigDeps generator" in c.out
    c.run("install app -c tools.cmake.cmakedeps:new=recipe_will_break")
    assert "WARN: Using the new CMakeConfigDeps generator" in c.out

    # conanfile.txt
    conanfile = textwrap.dedent("""
        [requires]
        dep/0.1
        [generators]
        CMakeConfigDeps
        """)
    c.save({"app/conanfile.txt": conanfile}, clean_first=True)
    c.run("install app", assert_error=True)
    assert "CMakeConfigDeps is being used in conanfile, but the conf " \
           "'tools.cmake.cmakedeps:new' is not enabled" in c.out
    c.run("install app -c tools.cmake.cmakedeps:new=will_break_next")
    assert "WARN: Using the new CMakeConfigDeps generator" in c.out
    c.run("install app -c tools.cmake.cmakedeps:new=recipe_will_break")
    assert "WARN: Using the new CMakeConfigDeps generator" in c.out


def test_system_wrappers():
    c = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan.tools.files import copy
        from conan import ConanFile
        class TestConan(ConanFile):
            name = "lib"
            version = "system"
            package_type = "shared-library"

            def package_info(self):
                self.cpp_info.includedirs = []
                self.cpp_info.libdirs = []
                self.cpp_info.system_libs = ["my_system_cool_lib"]
    """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")

    c.run(f"install --requires=lib/system -g CMakeConfigDeps "
          f"-c tools.cmake.cmakedeps:new={new_value}")
    cmake = c.load("lib-Targets-release.cmake")
    assert "add_library(lib::lib INTERFACE IMPORTED)" in cmake
    assert "set_property(TARGET lib::lib APPEND PROPERTY INTERFACE_LINK_LIBRARIES\n" \
           '             $<$<CONFIG:RELEASE>:my_system_cool_lib>)' in cmake


def test_autolink_pragma():
    """https://github.com/conan-io/conan/issues/10837"""
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def package_info(self):
                self.cpp_info.set_property("cmake_set_interface_link_directories", True)
        """)
    c.save({"conanfile.py": conanfile,
            "test_package/conanfile.py": GenConanfile().with_test("pass")
                                                       .with_settings("build_type")
                                                       .with_generator("CMakeDeps")})
    c.run("create . --name=pkg --version=0.1")
    assert "CMakeDeps: cmake_set_interface_link_directories is legacy, not necessary" in c.out
    c.run(f"create . --name=pkg --version=0.1 -c tools.cmake.cmakedeps:new={new_value}")
    assert "CMakeConfigDeps: cmake_set_interface_link_directories deprecated and invalid. " \
           "The package 'package_info()' must correctly define the (CPS) information" in c.out


def test_consuming_cpp_info_with_components_dependency_from_same_package():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def package_info(self):
                self.cpp_info.components["lib"].type = 'shared-library'
                self.cpp_info.components["lib_extended"].type = 'shared-library'
                self.cpp_info.components["lib_extended"].requires = ['lib']
        """)
    c.save({"conanfile.py": conanfile,
            "test_package/conanfile.py": GenConanfile().with_settings("build_type")
                                                       .with_test("pass")
                                                       .with_generator("CMakeDeps")})
    c.run(f"create . --name=pkg --version=0.1 -c tools.cmake.cmakedeps:new={new_value}")
    # it doesn't break
    assert "find_package(pkg)" in c.out


def test_consuming_cpp_info_with_components_dependency_from_other_package():
    c = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "0.1"
            def package_info(self):
                self.cpp_info.components["lib"].type = 'shared-library'
    """)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            requires = "dep/0.1"
            def package_info(self):
                self.cpp_info.components["lib"].type = 'shared-library'
                self.cpp_info.components["lib"].requires = ['dep::lib']
        """)
    c.save({"dep/conanfile.py": dep,
            "pkg/conanfile.py": conanfile,
            "pkg/test_package/conanfile.py": GenConanfile().with_settings("build_type")
                                                           .with_test("pass")
                                                           .with_generator("CMakeDeps")})
    c.run("create dep")
    c.run(f"create pkg --name=pkg --version=0.1 -c tools.cmake.cmakedeps:new={new_value}")
    # it doesn't break
    assert "find_package(pkg)" in c.out


def test_error_incorrect_component():
    # https://github.com/conan-io/conan/issues/18554
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            requires = "dep/0.1"
            def package_info(self):
                self.cpp_info.requires = ['dep::lib']
        """)
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
            "pkg/conanfile.py": conanfile,
            "pkg/test_package/conanfile.py": GenConanfile().with_settings("build_type")
                                                           .with_generator("CMakeDeps")
                                                           .with_test("pass")})
    c.run("create dep")
    c.run(f"create pkg --name=pkg --version=0.1 -c tools.cmake.cmakedeps:new={new_value}",
          assert_error=True)
    assert ("ERROR: Error in generator 'CMakeDeps': pkg/0.1 recipe cpp_info did .requires to "
            "'dep::lib' but component 'lib' not found in dep") in c.out


def test_consuming_cpp_info_transitively_by_requiring_root_component():
    c = TestClient()
    dependent_conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Dependent(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            name = 'dependent'
        """)

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            def requirements(self):
                self.requires('dependent/0.1')
            def package_info(self):
                self.cpp_info.type = 'shared-library'
                self.cpp_info.requires = ['dependent::dependent']
        """)
    test_package = textwrap.dedent("""
        from conan import ConanFile
        class TestPkg(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            generators = "VirtualRunEnv", "CMakeDeps"

            def requirements(self):
                self.requires(self.tested_reference_str)

            def test(self):
                pass
        """)
    c.save({"dependent/conanfile.py": dependent_conanfile,
            "main/conanfile.py": conanfile,
            "main/test_package/conanfile.py": test_package})
    c.run("create ./dependent/ --name=dependent --version=0.1 "
          f"-c tools.cmake.cmakedeps:new={new_value}")
    c.run(f"create ./main/ --name=pkg --version=0.1 -c tools.cmake.cmakedeps:new={new_value}")


def test_cmake_find_mode_deprecated():
    tc = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "0.1"
            def package_info(self):
                # Having both is ok as the user expects that config would
                # be generated nonetheless
                self.cpp_info.set_property("cmake_find_mode", "module")
        """)
    tc.save({"conanfile.py": dep})
    tc.run("create .")
    args = f"-g CMakeDeps -c tools.cmake.cmakedeps:new={new_value}"
    tc.run(f"install --requires=dep/0.1 {args}")
    assert "CMakeConfigDeps does not support module find mode"


def test_cmake_extra_dependencies():
    tc = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "0.1"
            def package_info(self):
                self.cpp_info.set_property("cmake_extra_dependencies", ["MyOpenMPI"])
                self.cpp_info.set_property("cmake_extra_interface_libs", ["MyOpenMPILib"])
        """)
    tc.save({"conanfile.py": dep})
    tc.run("create .")
    args = f"-g CMakeDeps -c tools.cmake.cmakedeps:new={new_value}"
    tc.run(f"install --requires=dep/0.1 {args}")
    dep = tc.load("dep-Targets-release.cmake")
    assert "find_dependency(MyOpenMPI REQUIRED )" in dep
    assert "set_property(TARGET dep::dep APPEND PROPERTY INTERFACE_LINK_LIBRARIES\n" \
           "             $<$<CONFIG:RELEASE>:MyOpenMPILib>)" in dep


def test_cmake_extra_dependencies_components():
    tc = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "0.1"
            def package_info(self):
                self.cpp_info.set_property("cmake_extra_dependencies", ["MyOpenMPI"])
                self.cpp_info.components["mycomp"].set_property("cmake_extra_interface_libs",
                                                                ["MyOpenMPILib"])
                self.cpp_info.components["mycomp"].libs = ["mycomplib"]
                self.cpp_info.components["mycomp"].type = "static-library"
                self.cpp_info.components["mycomp"].location = "lib/mycomp.a"
        """)
    tc.save({"conanfile.py": dep})
    tc.run("create .")
    args = f"-g CMakeDeps -c tools.cmake.cmakedeps:new={new_value}"
    tc.run(f"install --requires=dep/0.1 {args}")
    dep = tc.load("dep-Targets-release.cmake")
    assert "find_dependency(MyOpenMPI REQUIRED )" in dep
    assert "set_property(TARGET dep::mycomp APPEND PROPERTY INTERFACE_LINK_LIBRARIES\n" \
           "             $<$<CONFIG:RELEASE>:MyOpenMPILib>)" in dep


class TestRequiresToApp:
    def test_requires_to_application(self):
        c = TestClient()
        automake = GenConanfile("automake", "0.1").with_package_type("application")
        conanfile = (GenConanfile("libtool", "0.1").with_package_type("static-library")
                                                   .with_requirement("automake/0.1"))
        test_package = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.cmake import CMake

            class TestPkg(ConanFile):
                settings = "os", "compiler", "arch", "build_type"
                generators = "CMakeDeps", "CMakeToolchain"

                def requirements(self):
                    self.requires(self.tested_reference_str)

                def test(self):
                    pass
            """)

        c.save({"automake/conanfile.py": automake,
                "libtool/conanfile.py": conanfile,
                "libtool/test_package/conanfile.py": test_package})
        c.run("create automake")
        c.run(f"create libtool -c tools.cmake.cmakedeps:new={new_value}")
        targets = c.load("libtool/test_package/libtool-Targets-release.cmake")
        # The libtool shouldn't depend on the automake::automake target
        assert "automake::automake" not in targets

    def test_requires_to_application_component(self):
        c = TestClient()
        automake = textwrap.dedent("""
            from conan import ConanFile
            class Dependent(ConanFile):
                name = "automake"
                version = "0.1"
                package_type = "application"

                def package_info(self):
                    self.cpp_info.components["myapp"].exe = "myapp"
                    self.cpp_info.components["myapp"].location = "path/to/myapp"
                    self.cpp_info.components["mylibapp"].type = "header-library"
            """)

        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "libtool"
                version = "0.1"
                package_type = "static-library"

                def requirements(self):
                    self.requires('automake/0.1')
                def package_info(self):
                    self.cpp_info.requires = ["automake::mylibapp"]
            """)

        c.save({"automake/conanfile.py": automake,
                "libtool/conanfile.py": conanfile})
        c.run("create automake")

        c.run("create libtool")
        c.run("install --requires=libtool/0.1 -g CMakeDeps "
              f"-c tools.cmake.cmakedeps:new={new_value}")
        targets = c.load("libtool-Targets-release.cmake")
        # The libtool shouldn't depend on the automake::automake target
        assert "automake::automake" not in targets
        assert "# Requirement automake::mylibapp => Full link: True" in targets
        assert "$<$<CONFIG:RELEASE>:automake::mylibapp>" in targets

    def test_requires_from_library_component(self):
        c = TestClient()
        automake = GenConanfile("automake", "0.1").with_package_type("application")
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "libtool"
                version = "0.1"
                package_type = "static-library"

                def requirements(self):
                    self.requires('automake/0.1')
                def package_info(self):
                    self.cpp_info.components["mycomp"].requires = ["automake::automake"]
            """)

        c.save({"automake/conanfile.py": automake,
                "libtool/conanfile.py": conanfile})
        c.run("create automake")
        c.run("create libtool")
        c.run("install --requires=libtool/0.1 -g CMakeDeps "
              f"-c tools.cmake.cmakedeps:new={new_value}")
        targets = c.load("libtool-Targets-release.cmake")
        # The libtool shouldn't depend on the automake::automake target
        assert "automake::automake" not in targets

    def test_requires_from_library_component_to_app_component(self):
        c = TestClient()
        automake = textwrap.dedent("""
            from conan import ConanFile
            class Dependent(ConanFile):
                name = "automake"
                version = "0.1"

                def package_info(self):
                    self.cpp_info.components["myapp"].exe = "myapp"
                    self.cpp_info.components["myapp"].location = "path/to/myapp"
                    self.cpp_info.components["mylibapp"].type = "header-library"
            """)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "libtool"
                version = "0.1"
                package_type = "static-library"

                def requirements(self):
                    self.requires('automake/0.1')
                def package_info(self):
                    self.cpp_info.components["mycomp"].requires = ["automake::myapp"]
            """)

        c.save({"automake/conanfile.py": automake,
                "libtool/conanfile.py": conanfile})
        c.run("create automake")
        c.run("create libtool")
        c.run("install --requires=libtool/0.1 -g CMakeDeps "
              f"-c tools.cmake.cmakedeps:new={new_value}")
        targets = c.load("libtool-Targets-release.cmake")
        # The libtool shouldn't depend on the automake::automake target
        assert "automake::myapp" not in targets
        assert "automake::automake" not in targets


def test_alias_cmakedeps_set_property():
    tc = TestClient()
    tc.save({"dep/conanfile.py": textwrap.dedent("""

        from conan import ConanFile
        class Dep(ConanFile):
            name = "dep"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            def package_info(self):
                self.cpp_info.components["mycomp"].set_property("cmake_target_name",
                                                                "dep::mycomponent")
        """),
             "conanfile.py": textwrap.dedent("""
             from conan import ConanFile
             from conan.tools.cmake import CMakeDeps, CMake
             class Pkg(ConanFile):
                name = "pkg"
                version = "1.0"
                settings = "os", "compiler", "build_type", "arch"

                requires = "dep/1.0"

                def generate(self):
                    deps = CMakeDeps(self)
                    deps.set_property("dep", "cmake_target_aliases", ["alias", "dep::other_name"])
                    deps.set_property("dep::mycomp", "cmake_target_aliases",
                                      ["component_alias", "dep::my_aliased_component"])
                    deps.generate()
             """)})
    tc.run("create dep")
    tc.run(f"install . -c tools.cmake.cmakedeps:new={new_value}")
    targets_data = tc.load('dep-Targets-release.cmake')
    assert "add_library(dep::dep" in targets_data
    assert "add_library(alias" in targets_data
    assert "add_library(dep::other_name" in targets_data

    assert "add_library(component_alias" in targets_data
    assert "add_library(dep::my_aliased_component" in targets_data


def test_package_info_extra_variables():
    """ Test extra_variables property - This just shows that it works,
    there are tests for cmaketoolchain that check the actual behavior
    of parsing the variables"""
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def package_info(self):
                self.cpp_info.set_property("cmake_extra_variables", {"FOO": 42,
                                           "BAR": 42,
                                           "CMAKE_GENERATOR_INSTANCE":
                                                 "${GENERATOR_INSTANCE}/buildTools/",
                                           "CACHE_VAR_DEFAULT_DOC": {"value": "hello world",
                                                                     "cache": True, "type": "PATH"}})
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    client.run(f"install --requires=pkg/0.1 -g CMakeDeps -c tools.cmake.cmakedeps:new={new_value} "
               """-c tools.cmake.cmaketoolchain:extra_variables="{'BAR': 9}" """)
    target = client.load("pkg-config.cmake")
    assert 'set(BAR' not in target
    assert 'set(CMAKE_GENERATOR_INSTANCE "${GENERATOR_INSTANCE}/buildTools/")' in target
    assert 'set(FOO 42)' in target
    assert 'set(CACHE_VAR_DEFAULT_DOC "hello world" CACHE PATH "CACHE_VAR_DEFAULT_DOC")' in target


def test_target_defines_only():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def package_info(self):
                self.cpp_info.components["base"].includedirs = []
                self.cpp_info.components["base"].defines = ["FOO=1"]
                self.cpp_info.components["comp"].includedirs = ["include"]
                self.cpp_info.components["comp"].requires = ["base"]
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    client.run(f"install --requires=pkg/0.1 -g CMakeDeps -c tools.cmake.cmakedeps:new={new_value}")
    target = client.load("pkg-Targets-release.cmake")
    assert 'add_library(pkg::base INTERFACE IMPORTED)' in target
    assert "# Requirement pkg::base => Full link: True" in target



import os
import stat

from conan.tools.files import replace_in_file, unzip
from conan.test.utils.mocks import ConanFileMock

from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import load, save


class TestConanfileTools:

    def test_save_append(self):
        # https://github.com/conan-io/conan/issues/2841 (regression)
        client = TestClient()
        conanfile = """from conan import ConanFile
from conan.tools.files import save
class Pkg(ConanFile):
    def source(self):
        save(self, "myfile.txt", "Hello", append=True)
"""
        client.save({"conanfile.py": conanfile,
                     "myfile.txt": "World"})
        client.run("source .")
        assert "WorldHello" == client.load("myfile.txt")

    def test_untar(self):
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "example.txt")
        save(file_path, "Hello world!")
        tar_path = os.path.join(tmp_dir, "sample.tar")
        try:
            old_path = os.getcwd()
            os.chdir(tmp_dir)
            import tarfile
            tar = tarfile.open(tar_path, "w")
            tar.add("example.txt")
            tar.close()
        finally:
            os.chdir(old_path)
        output_dir = os.path.join(tmp_dir, "output_dir")
        unzip(ConanFileMock(), tar_path, output_dir)
        content = load(os.path.join(output_dir, "example.txt"))
        assert content == "Hello world!"

    def test_replace_in_file(self):
        tmp_dir = temp_folder()
        text_file = os.path.join(tmp_dir, "text.txt")
        save(text_file, "ONE TWO THREE")
        replace_in_file(ConanFileMock(), text_file, "ONE TWO THREE", "FOUR FIVE SIX")
        assert load(text_file) == "FOUR FIVE SIX"

    def test_replace_in_file_readonly(self):
        """
        If we try to replace_in_file a read only file it should raise a PermissionError
        """
        tmp_dir = temp_folder()
        text_file = os.path.join(tmp_dir, "text.txt")
        save(text_file, "ONE TWO THREE")

        os.chmod(text_file,
                 os.stat(text_file).st_mode & ~(stat.S_IWRITE | stat.S_IWGRP | stat.S_IWOTH))

        try:
            replace_in_file(ConanFileMock(), text_file, "ONE TWO THREE", "FOUR FIVE SIX")
            assert False, "Expected PermissionError"
        except PermissionError:
            pass



import os
import shutil
from unittest import mock

import pytest

from conan.tools.files import ftp_download, download, get
from conan.internal.errors import AuthenticationException
from conan.errors import ConanException
from conan.test.utils.file_server import TestFileServer
from conan.test.utils.mocks import ConanFileMock, RedirectedTestOutput
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import redirect_output, TestRequester
from conan.internal.util.files import save, load, chdir, mkdir


@mock.patch('ftplib.FTP', autospec=True)
class TestFTP:

    def test_ftp_auth(self, ftp_mock):
        ftp_object = ftp_mock.return_value
        filename = "/pub/example/readme2.txt"
        with chdir(temp_folder()):  # To not pollute the project
            ftp_download(None, "test.rebex.net", filename, "demo", "password")
        ftp_mock.assert_called_with('test.rebex.net')
        assert ftp_object.login.called
        ftp_object.cwd.assert_called_with('/pub/example')

    def test_ftp_invalid_path(self, ftp_mock):
        ftp_object = ftp_mock.return_value
        ftp_object.cwd.side_effect = Exception("550 The system cannot find the file specified.")
        with pytest.raises(ConanException) as exc:
            ftp_download(None, "test.rebex.net", "/path/invalid-file", "demo", "password")
        assert "550 The system cannot find the file specified." in str(exc.value)
        assert not os.path.exists("invalid-file")

    def test_ftp_invalid_auth(self, ftp_mock):
        ftp_object = ftp_mock.return_value
        ftp_object.login.side_effect = Exception("530 User cannot log in.")
        with pytest.raises(ConanException) as exc:
            ftp_download(None, "test.rebex.net", "readme.txt", "demo", "invalid")
        assert "530 User cannot log in." in str(exc.value)
        assert not os.path.exists("readme.txt")


class TestDownload:
    @pytest.fixture()
    def _manual(self):
        file_server = TestFileServer()
        save(os.path.join(file_server.store, "manual.html"), "this is some content")
        conanfile = ConanFileMock()
        conanfile._conan_helpers.requester = TestRequester({"file_server": file_server})
        return conanfile, file_server

    def test_download(self, _manual):
        conanfile, file_server = _manual
        dest = os.path.join(temp_folder(), "manual.html")
        download(conanfile, file_server.fake_url + "/manual.html", dest, retry=3, retry_wait=0)
        content = load(dest)
        assert content == "this is some content"

        # Can re-download
        download(conanfile, file_server.fake_url + "/manual.html", dest, retry=3, retry_wait=0)
        content = load(dest)
        assert content == "this is some content"

    def test_download_iterate_url(self, _manual):
        conanfile, file_server = _manual
        dest = os.path.join(temp_folder(), "manual.html")
        output = RedirectedTestOutput()
        with redirect_output(output):
            download(conanfile, ["invalid", file_server.fake_url + "/manual.html"], dest,
                     retry=3, retry_wait=0)
        content = load(dest)
        assert content == "this is some content"
        assert "Trying another mirror." in str(output)

    def test_download_forbidden(self, _manual):
        conanfile, file_server = _manual
        dest = os.path.join(temp_folder(), "manual.html")
        # Not authorized
        with pytest.raises(AuthenticationException) as exc:
            download(conanfile, file_server.fake_url + "/forbidden", dest)
        assert "403 Forbidden" in str(exc.value)

    def test_download_unauthorized_no_credentials(self, _manual):
        conanfile, file_server = _manual
        dest = os.path.join(temp_folder(), "manual.html")
        # Not authorized without credentials
        with pytest.raises(AuthenticationException) as exc:
            download(conanfile, file_server.fake_url + "/basic-auth/manual.html", dest)
        assert "401 Unauthorized" in str(exc.value)
        assert "Not authorized" in str(exc.value)

    def test_download_unauthorized_literal_none_credentials(self, _manual):
        conanfile, file_server = _manual
        dest = os.path.join(temp_folder(), "manual.html")
        # os.getenv('SOME_UNSET_VARIABLE') causes source_credentials.json to contain
        # "None" as a string literal, causing auth to be ("None", "None") or with
        # a "None" token.
        auth = ("None", "None")
        with pytest.raises(AuthenticationException) as exc:
            download(conanfile, file_server.fake_url + "/basic-auth/manual.html", dest, auth=auth)
        assert "401 Unauthorized" in str(exc.value)
        assert "Bad credentials" in str(exc.value)

    def test_download_authorized(self, _manual):
        conanfile, file_server = _manual
        dest = os.path.join(temp_folder(), "manual.html")
        with pytest.raises(AuthenticationException):
            download(conanfile, file_server.fake_url + "/basic-auth/manual.html",
                     dest, auth=("user", "wrong"), retry=0, retry_wait=0)

        # Authorized
        download(conanfile, file_server.fake_url + "/basic-auth/manual.html", dest,
                 auth=("user", "password"), retry=0, retry_wait=0)

        # Authorized using headers
        download(conanfile, file_server.fake_url + "/basic-auth/manual.html", dest,
                 headers={"Authorization": "Bearer password"}, retry=0, retry_wait=0)

    def test_download_retries_errors(self):
        conanfile = ConanFileMock()
        conanfile._conan_helpers.requester = TestRequester({})

        with pytest.raises(ConanException):
            output = RedirectedTestOutput()
            with redirect_output(output):
                download(conanfile, "http://fakesomething", "path", retry=2, retry_wait=0.1)
        assert str(output).count("Waiting 0.1 seconds to retry...") == 2

    def test_download_retries_500_errors(self, _manual):
        conanfile, file_server = _manual
        with pytest.raises(ConanException):
            output = RedirectedTestOutput()
            with redirect_output(output):
                download(conanfile, file_server.fake_url + "/internal_error", "path",
                         retry=2, retry_wait=0.1)
        assert str(output).count("Waiting 0.1 seconds to retry...") == 2

    def test_download_no_retries_errors(self):
        # Not found error will not retry
        file_server = TestFileServer()
        conanfile = ConanFileMock()
        conanfile._conan_helpers.requester = TestRequester({"file_server": file_server})
        file_path = os.path.join(temp_folder(), "file.txt")
        with pytest.raises(ConanException):
            download(conanfile, file_server.fake_url + "/manual.html", file_path,
                     retry=2, retry_wait=0)
        assert "Waiting" not in str(conanfile.output)
        assert "retry" not in str(conanfile.output)

    def test_download_localfile(self):
        conanfile = ConanFileMock()

        file_location = os.path.join(temp_folder(), "file.txt")
        save(file_location, "this is some content")

        file_location = file_location.lstrip("/")
        file_url = f"file:///{file_location}"
        file_md5 = "736db904ad222bf88ee6b8d103fceb8e"

        dest = os.path.join(temp_folder(), "downloaded_file.txt")
        download(conanfile, file_url, dest, md5=file_md5)
        content = load(dest)
        assert "this is some content" == content

    def test_download_localfile_notfound(self):
        conanfile = ConanFileMock()

        file_url = "file:///path/to/missing/file.txt"
        dest = os.path.join(temp_folder(), "file.txt")

        with pytest.raises(FileNotFoundError) as exc:
            download(conanfile, file_url, dest)

        assert "No such file" in str(exc.value)


class TestGet:

    @pytest.fixture()
    def _my_zip(self):
        tmp_folder = temp_folder()
        file_path = os.path.join(tmp_folder, "sample.tar.gz")
        test_folder = os.path.join(tmp_folder, "test_folder")
        zipped_file = os.path.join(test_folder, "myfile.txt")
        save(zipped_file, "myfile contents!")
        import tarfile
        tar_file = tarfile.open(file_path, "w:gz")
        tar_file.add(test_folder, "test_folder")
        tar_file.add(zipped_file, "test_folder/myfile.txt")
        tar_file.close()
        assert (os.path.exists(file_path))

        file_server = TestFileServer()
        shutil.copy2(file_path, file_server.store)
        conanfile = ConanFileMock()
        conanfile._conan_helpers.requester = TestRequester({"file_server": file_server})

        return conanfile, file_server

    def test_get_tgz(self, _my_zip):
        conanfile, file_server = _my_zip
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            get(conanfile, file_server.fake_url + "/sample.tar.gz", retry=0, retry_wait=0)
            assert load("test_folder/myfile.txt") == "myfile contents!"

    def test_get_tgz_strip_root(self, _my_zip):
        conanfile, file_server = _my_zip
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            get(conanfile, file_server.fake_url + "/sample.tar.gz", retry=0, retry_wait=0,
                strip_root=True)
            assert load("myfile.txt") == "myfile contents!"


class TestGetGz:
    @pytest.fixture()
    def _my_gz(self):
        tmp = temp_folder()
        filepath = os.path.join(tmp, "test.txt.gz")
        import gzip
        with gzip.open(filepath, "wb") as f:
            f.write(b"hello world zipped!")

        file_server = TestFileServer()
        folder = os.path.join(file_server.store, "gz")
        mkdir(folder)
        shutil.copy2(filepath, folder)
        conanfile = ConanFileMock()
        conanfile._conan_helpers.requester = TestRequester({"file_server": file_server})
        return conanfile, file_server

    def test_get_gunzip(self, _my_gz):
        conanfile, file_server = _my_gz
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            get(conanfile, file_server.fake_url + "/gz/test.txt.gz", retry=0, retry_wait=0)
            assert load("test.txt") == "hello world zipped!"

    def test_get_gunzip_destination(self, _my_gz):
        conanfile, file_server = _my_gz
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            get(conanfile, file_server.fake_url + "/gz/test.txt.gz", destination="myfile.doc",
                retry=0, retry_wait=0)
            assert load("myfile.doc") == "hello world zipped!"

    def test_get_gunzip_destination_subfolder(self, _my_gz):
        conanfile, file_server = _my_gz
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            get(conanfile, file_server.fake_url + "/gz/test.txt.gz",
                destination="sub/myfile.doc", retry=0, retry_wait=0)
            assert load("sub/myfile.doc") == "hello world zipped!"

    def test_get_filename_error(self):
        # Test: File name cannot be deduced from '?file=1'
        file_server = TestFileServer()
        conanfile = ConanFileMock()
        conanfile._conan_helpers.requester = TestRequester({"file_server": file_server})

        with pytest.raises(ConanException) as error:
            get(conanfile, file_server.fake_url + "?file=1")
        assert "Cannot deduce file name from the url" in str(error.value)



import os
import platform

import pytest

from conan.test.assets.genconanfile import GenConanfile


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
def test_transitive_headers_not_public(transitive_libraries):
    c = transitive_libraries
    c.save({"conanfile.py": GenConanfile().with_settings("build_type", "arch")
                                          .with_generator("MSBuildDeps").with_requires("engine/1.0")},
           clean_first=True)

    c.run("install .")
    matrix_data = c.load("conan_matrix_vars_release_x64.props")
    assert "<ConanmatrixIncludeDirectories></ConanmatrixIncludeDirectories>" in matrix_data
    assert "<ConanmatrixLibraryDirectories>$(ConanmatrixRootFolder)/lib;" \
           "</ConanmatrixLibraryDirectories>" in matrix_data
    assert "<ConanmatrixLibraries>matrix.lib;</ConanmatrixLibraries>" in matrix_data


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
def test_shared_requires_static(transitive_libraries):
    c = transitive_libraries
    c.save({"conanfile.py": GenConanfile().with_settings("build_type", "arch")
           .with_generator("MSBuildDeps").with_requires("engine/1.0")},
           clean_first=True)

    c.run("install . -o engine*:shared=True")
    assert not os.path.exists(os.path.join(c.current_folder, "conan_matrix_vars_release_x64.props"))
    engine_data = c.load("conan_engine_release_x64.props")
    # No dependency to matrix, it has been skipped as unnecessary
    assert "conan_matrix.props" not in engine_data



import json
import os
import re

import textwrap
from textwrap import dedent

from conan.internal.paths import CONANFILE
from conan.test.utils.tools import TestClient, GenConanfile
from conan.internal.util.files import load


class TestExportPkg:

    def test_dont_touch_server(self):
        # https://github.com/conan-io/conan/issues/3432
        client = TestClient(servers={"default": None},
                            requester_class=None, inputs=["admin", "password"])

        client.save({"conanfile.py": GenConanfile().with_name("pkg").with_version("0.1")})
        client.run("install .")
        client.run("export-pkg . --user=lasote --channel=stable ")

    def test_transitive_without_settings(self):
        # https://github.com/conan-io/conan/issues/3367
        client = TestClient()
        client.save({"pkgc/conanfile.py": GenConanfile("pkgc", "0.1"),
                     "pkgb/conanfile.py": GenConanfile("pkgb", "0.1").with_requires("pkgc/0.1"),
                     "pkga/conanfile.py": GenConanfile("pkga", "0.1").with_requires("pkgb/0.1")})
        client.run("create pkgc")
        client.run("create pkgb")

        client.run("build pkga -bf=build")
        client.run("export-pkg pkga ")
        package_id = re.search(r"Packaging to (\S+)", str(client.out)).group(1)
        assert f"conanfile.py (pkga/0.1): Package '{package_id}' created" in client.out

        # we can export-pkg without the dependencies binaries if we need to optimize
        client.run("remove pkgc*:* -c")
        client.run("remove pkgb*:* -c")
        client.run("export-pkg pkga --skip-binaries")
        package_id = re.search(r"Packaging to (\S+)", str(client.out)).group(1)
        assert f"conanfile.py (pkga/0.1): Package '{package_id}' created" in client.out

    def test_package_folder_errors(self):
        # https://github.com/conan-io/conan/issues/2350
        client = TestClient()
        client.save({CONANFILE: GenConanfile()})
        client.run("export-pkg --name=hello --version=0.1 --user=lasote --channel=stable")
        assert ("conanfile.py (hello/0.1@lasote/stable): package(): "
                "WARN: No files in this package!") in client.out

    def test_options(self):
        # https://github.com/conan-io/conan/issues/2242
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Hello(ConanFile):
                name = "hello"
                version = "0.1"
                options = { "optionOne": [True, False, 123] }
                default_options =  {"optionOne": True}
            """)
        client = TestClient()
        client.save({CONANFILE: conanfile})
        client.run("export-pkg .")
        client.run("list hello/0.1:*")
        assert "optionOne: True" in client.out
        assert "optionOne: False" not in client.out
        assert "optionOne: 123" not in client.out
        client.run("export-pkg . -o optionOne=False")
        client.run("list hello/0.1:*")
        assert "optionOne: True" in client.out
        assert "optionOne: False" in client.out
        assert "optionOne: 123" not in client.out
        client.run("export-pkg . -o hello/*:optionOne=123")
        client.run("list hello/0.1:*")
        assert "optionOne: True" in client.out
        assert "optionOne: False" in client.out
        assert "optionOne: 123" in client.out

    def test_profile_environment(self):
        # https://github.com/conan-io/conan/issues/4832
        conanfile = dedent("""
            import os
            from conan import ConanFile
            from conan.tools.env import VirtualBuildEnv
            class helloPythonConan(ConanFile):
                def package(self):
                    build_env = VirtualBuildEnv(self).vars()
                    with build_env.apply():
                        self.output.info("ENV-VALUE: %s!!!" % os.getenv("MYCUSTOMVAR"))
            """)
        profile = dedent("""
            [buildenv]
            MYCUSTOMVAR=MYCUSTOMVALUE
            """)
        client = TestClient()
        client.save({CONANFILE: conanfile,
                     "myprofile": profile})
        client.run("export-pkg . --name=hello --version=0.1 --user=lasote --channel=stable "
                   " -pr=myprofile")
        assert "conanfile.py (hello/0.1@lasote/stable): ENV-VALUE: MYCUSTOMVALUE!!!" in client.out

    def test_build_folders(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.files import copy
            class TestConan(ConanFile):
                name = "hello"
                version = "0.1"
                settings = "os"

                def package(self):
                    copy(self, "*.h", os.path.join(self.source_folder, "include"),
                         os.path.join(self.package_folder, "inc"))
                    copy(self, "*.lib", os.path.join(self.build_folder, "lib"),
                         os.path.join(self.package_folder, "lib"))
            """)
        client.save({CONANFILE: conanfile,
                     "include/header.h": "//Windows header",
                     "include/header.txt": "",
                     "libs/what": "",
                     "lib/hello.lib": "My Lib",
                     "lib/bye.txt": ""}, clean_first=True)
        client.run("export-pkg . --user=lasote --channel=stable -s os=Windows -vvv")
        assert "copy(pattern=*.h) copied 1 files" in client.out
        assert "copy(pattern=*.lib) copied 1 files" in client.out
        package_folder = client.created_layout().package()
        inc = os.path.join(package_folder, "inc")
        assert os.listdir(inc) == ["header.h"]
        assert load(os.path.join(inc, "header.h")) == "//Windows header"
        lib = os.path.join(package_folder, "lib")
        assert os.listdir(lib) == ["hello.lib"]
        assert load(os.path.join(lib, "hello.lib")) == "My Lib"

    def test_default_source_folder(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.files import copy
            class TestConan(ConanFile):

                def package(self):
                    copy(self, "*.h", os.path.join(self.source_folder, "src"),
                         os.path.join(self.package_folder, "include"))
                    copy(self, "*.lib", self.build_folder, os.path.join(self.package_folder, "lib"),
                         keep_path=False)
            """)
        client.save({CONANFILE: conanfile,
                     "src/header.h": "contents",
                     "build/lib/hello.lib": "My Lib"})
        client.run("export-pkg . --name=hello --version=0.1 --user=lasote --channel=stable "
                   "-s os=Windows")

        package_folder = client.created_layout().package()
        header = os.path.join(package_folder, "include/header.h")
        assert os.path.exists(header)

        hello_path = os.path.join(package_folder, "lib", "hello.lib")
        assert os.path.exists(hello_path)

    def test_build_source_folders(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.files import copy
            class TestConan(ConanFile):
                settings = "os"
                name = "hello"
                version = "0.1"

                def layout(self):
                    self.folders.build = "build"
                    self.folders.source = "src"

                def package(self):
                    copy(self, "*.h", os.path.join(self.source_folder, "include"),
                         os.path.join(self.package_folder, "inc"))
                    copy(self, "*.lib", os.path.join(self.build_folder, "lib"),
                         os.path.join(self.package_folder, "lib"))
            """)
        client.save({CONANFILE: conanfile,
                     "src/include/header.h": "//Windows header",
                     "src/include/header.txt": "",
                     "build/libs/what": "",
                     "build/lib/hello.lib": "My Lib",
                     "build/lib/bye.txt": ""})
        client.run("export-pkg . --user=lasote --channel=stable -s os=Windows")

        package_folder = client.created_layout().package()
        inc = os.path.join(package_folder, "inc")
        assert os.listdir(inc) == ["header.h"]
        assert load(os.path.join(inc, "header.h")) == "//Windows header"
        lib = os.path.join(package_folder, "lib")
        assert os.listdir(lib) == ["hello.lib"]
        assert load(os.path.join(lib, "hello.lib")) == "My Lib"

    def test_partial_references(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import copy
            class TestConan(ConanFile):
                name = "hello"
                version = "0.1"
                settings = "os"

                def package(self):
                    copy(self, "*", self.source_folder, self.package_folder)
            """)
        # Partial reference is ok
        client.save({CONANFILE: conanfile, "file.txt": "txt contents"})
        client.run("export-pkg . --user=conan --channel=stable")
        assert ("conanfile.py (hello/0.1@conan/stable): package(): "
                "Packaged 1 '.txt' file: file.txt") in client.out

        # Specify different name or version is not working
        client.run("export-pkg . --name=lib", assert_error=True)
        assert "ERROR: Package recipe with name lib!=hello" in client.out

        client.run("export-pkg . --version=1.1", assert_error=True)
        assert "ERROR: Package recipe with version 1.1!=0.1" in client.out

        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import copy
            class TestConan(ConanFile):
                settings = "os"

                def package(self):
                    copy(self, "*", self.source_folder, self.package_folder)
            """)
        # Partial reference is ok
        client.save({CONANFILE: conanfile, "file.txt": "txt contents"})
        client.run("export-pkg . --name=anyname --version=1.222 --user=conan --channel=stable")
        assert ("conanfile.py (anyname/1.222@conan/stable): package(): "
                "Packaged 1 '.txt' file: file.txt") in client.out

    def test_with_deps(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . --name=hello --version=0.1 --user=lasote --channel=stable")
        conanfile = GenConanfile().with_name("hello1").with_version("0.1")\
                                  .with_import("from conan.tools.files import copy, collect_libs") \
                                  .with_require("hello/0.1@lasote/stable")

        conanfile = str(conanfile) + """\n    def package_info(self):
        self.cpp_info.libs = collect_libs(self)
    def layout(self):
        self.folders.build = "Release_x86"
    def package(self):
        copy(self, "*", self.source_folder, self.package_folder)
        copy(self, "*", self.build_folder, self.package_folder)
        """
        client.save({"conanfile.py": conanfile}, clean_first=True)
        client.save({"Release_x86/lib/libmycoollib.a": ""})
        settings = ('-s os=Windows -s compiler=gcc -s compiler.version=4.9 '
                    '-s compiler.libcxx=libstdc++ -s build_type=Release -s arch=x86')
        client.run("export-pkg . --name=hello1 --version=0.1 --user=lasote --channel=stable %s"
                   % settings)

        consumer = (GenConanfile().with_requires("hello1/0.1@lasote/stable")
                    .with_settings("os", "build_type"))

        client.save({CONANFILE: consumer}, clean_first=True)
        client.run("install conanfile.py -g CMakeDeps")
        assert "hello/0.1@lasote/stable: Already installed!" in client.out
        assert "hello1/0.1@lasote/stable: Already installed!" in client.out

        cmakeinfo = client.load("hello1-release-data.cmake")
        assert "set(hello1_LIBS_RELEASE mycoollib)" in cmakeinfo

    def test_export_pkg_version_type(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Hello(ConanFile):
                def requirements(self):
                    assert type(self.version) is str, "Version should be a string"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=hello --version=0.1")
        # This used to trigger the assertion error
        client.run("export-pkg . --name=hello --version=0.1")

    def test_export_pkg_json(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("pkg", "0.1")})

        # Wrong folders
        client.run("export-pkg . --format=json", redirect_stdout="exported.json")
        graph = json.loads(client.load("exported.json"))
        node = graph["graph"]["nodes"]["0"]
        assert "pkg/0.1" in node["ref"]
        # https://github.com/conan-io/conan/issues/15041
        assert "da39a3ee5e6b4b0d3255bfef95601890afd80709" == node["package_id"]
        assert "485dad6cb11e2fa99d9afbe44a57a164" == node["rrev"]
        assert "0ba8627bd47edc3a501e8f0eb9a79e5e" == node["prev"]
        assert "Build" == node["binary"]
        assert node["rrev_timestamp"] is not None
        assert node["prev_timestamp"] is not None

        # Make sure the exported json file can be used for ``conan lsit --graph`` input to upload
        client.run("list --graph=exported.json -gb=build --format=json")
        pkglist = json.loads(client.stdout)
        revs = pkglist["Local Cache"]["pkg/0.1"]["revisions"]["485dad6cb11e2fa99d9afbe44a57a164"]
        assert "da39a3ee5e6b4b0d3255bfef95601890afd80709" in revs["packages"]

    def test_export_pkg_no_ref(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.files import copy
            class TestConan(ConanFile):
                name = "hello"
                version = "0.1"

                def package(self):
                    copy(self, "*.h", os.path.join(self.source_folder, "src"),
                         os.path.join(self.package_folder, "include"))
            """)
        client.save({CONANFILE: conanfile,
                     "src/header.h": "contents"})
        client.run("export-pkg . -s os=Windows")
        package_folder = client.created_layout().package()
        header = os.path.join(package_folder, "include/header.h")
        assert os.path.exists(header)


def test_build_policy_never():
    client = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import copy
        class TestConan(ConanFile):
            build_policy = "never"

            def package(self):
                copy(self, "*.h", os.path.join(self.source_folder, "src"),
                     os.path.join(self.package_folder, "include"))
        """)
    client.save({CONANFILE: conanfile,
                 "src/header.h": "contents"})
    client.run("export-pkg . --name=pkg --version=1.0")
    assert "conanfile.py (pkg/1.0): package(): Packaged 1 '.h' file: header.h" in client.out
    # check for https://github.com/conan-io/conan/issues/10736
    client.run("export-pkg . --name=pkg --version=1.0")
    assert "conanfile.py (pkg/1.0): package(): Packaged 1 '.h' file: header.h" in client.out
    client.run("install --requires=pkg/1.0@ --build='*'")
    client.assert_listed_require({"pkg/1.0": "Cache"})
    assert "conanfile.py (pkg/1.0): Calling build()" not in client.out


def test_build_policy_never_missing():
    # https://github.com/conan-io/conan/issues/9798
    client = TestClient()
    client.save({"conanfile.py": GenConanfile().with_class_attribute('build_policy = "never"'),
                 "consumer.txt": "[requires]\npkg/1.0"})
    client.run("export . --name=pkg --version=1.0")
    client.run("install --requires=pkg/1.0@ --build='*'", assert_error=True)
    assert "ERROR: Missing binary: pkg/1.0" in client.out

    client.run("install --requires=pkg/1.0@ --build=missing", assert_error=True)
    assert "ERROR: Missing binary: pkg/1.0" in client.out


def test_export_pkg_json_formatter():
    """
    Tests the ``conan export-pkg . -f json`` result
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class MyTest(ConanFile):
            name = "pkg"
            version = "0.2"

            def package_info(self):
                self.cpp_info.libs = ["pkg"]
                self.cpp_info.includedirs = ["path/includes/pkg", "other/include/path/pkg"]
                self.cpp_info.libdirs = ["one/lib/path/pkg"]
                self.cpp_info.defines = ["pkg_onedefinition", "pkg_twodefinition"]
                self.cpp_info.cflags = ["pkg_a_c_flag"]
                self.cpp_info.cxxflags = ["pkg_a_cxx_flag"]
                self.cpp_info.sharedlinkflags = ["pkg_shared_link_flag"]
                self.cpp_info.exelinkflags = ["pkg_exe_link_flag"]
                self.cpp_info.sysroot = "/path/to/folder/pkg"
                self.cpp_info.frameworks = ["pkg_oneframework", "pkg_twoframework"]
                self.cpp_info.system_libs = ["pkg_onesystemlib", "pkg_twosystemlib"]
                self.cpp_info.frameworkdirs = ["one/framework/path/pkg"]
                self.cpp_info.set_property("pkg_config_name", "pkg_other_name")
                self.cpp_info.set_property("pkg_config_aliases", ["pkg_alias1", "pkg_alias2"])
                self.cpp_info.components["cmp1"].libs = ["libcmp1"]
                self.cpp_info.components["cmp1"].set_property("pkg_config_name", "compo1")
                self.cpp_info.components["cmp1"].set_property("pkg_config_aliases", ["compo1_alias"])
                self.cpp_info.components["cmp1"].sysroot = "/another/sysroot"
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.save({"conanfile.py": GenConanfile().with_name("hello").with_version("0.1")
                .with_require("pkg/0.2")}, clean_first=True)
    client.run("export-pkg . -f json")
    info = json.loads(client.stdout)
    nodes = info["graph"]["nodes"]
    hello_pkg_ref = 'hello/0.1#18d5440ae45afc4c36139a160ac071c7'
    pkg_pkg_ref = 'pkg/0.2#926714b5fb0a994f47ec37e071eba1da'
    hello_cpp_info = pkg_cpp_info = None
    for n in nodes.values():
        ref = n["ref"]
        if ref == hello_pkg_ref:
            assert n['binary'] == "Build"
            hello_cpp_info = n['cpp_info']
        elif ref == pkg_pkg_ref:
            assert n['binary'] == "Cache"
            pkg_cpp_info = n['cpp_info']
    assert hello_cpp_info and pkg_cpp_info
    # hello/0.1 cpp_info
    assert hello_cpp_info['root']["libs"] is None
    assert len(hello_cpp_info['root']["bindirs"]) == 1
    assert len(hello_cpp_info['root']["libdirs"]) == 1
    assert hello_cpp_info['root']["sysroot"] is None
    assert hello_cpp_info['root']["properties"] is None
    # pkg/0.2 cpp_info
    # root info

    assert pkg_cpp_info['root']["libs"] == ['pkg']
    assert len(pkg_cpp_info['root']["bindirs"]) == 1
    assert len(pkg_cpp_info['root']["libdirs"]) == 1
    assert pkg_cpp_info['root']["sysroot"] == '/path/to/folder/pkg'
    assert pkg_cpp_info['root']["system_libs"] == ['pkg_onesystemlib', 'pkg_twosystemlib']
    assert pkg_cpp_info['root']['cflags'] == ['pkg_a_c_flag']
    assert pkg_cpp_info['root']['cxxflags'] == ['pkg_a_cxx_flag']
    assert pkg_cpp_info['root']['defines'] == ['pkg_onedefinition', 'pkg_twodefinition']
    assert pkg_cpp_info['root']["properties"] == {'pkg_config_name': 'pkg_other_name',
                                                  'pkg_config_aliases': ['pkg_alias1', 'pkg_alias2']}
    # component info
    assert pkg_cpp_info["cmp1"]["libs"] == ["libcmp1"]


def test_export_pkg_dont_update_src():
    """
    There was a bug in 1.X and sources were not updated correctly in export-pkg
    close https://github.com/conan-io/conan/issues/6041
    """
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import load
        class Hello(ConanFile):
            name = "hello"
            version = "0.1"
            exports_sources = "*.cpp"
            def build(self):
                content = load(self, "src/hello.cpp")
                self.output.info("CONTENT: {}".format(content))
        """)

    c.save({"conanfile.py": conanfile,
            "src/hello.cpp": "old code!"})
    c.run("install .")
    c.run("build .")
    c.run("export-pkg .")
    c.run("install --requires=hello/0.1@ --build=hello*")
    assert "hello/0.1: CONTENT: old code!" in c.out
    # Now locally change the source code
    c.save({"src/hello.cpp": "updated code!"})
    c.run("install .")
    c.run("build .")
    c.run("export-pkg .")
    c.run("install --requires=hello/0.1@ --build=hello*")
    assert "hello/0.1: CONTENT: updated code!" in c.out


def test_negate_tool_requires():
    c = TestClient()
    profile = textwrap.dedent("""
        [tool_requires]
        !mypkg/*:cmake/3.24
        """)
    c.save({"myprofile": profile,
            "conanfile.py": GenConanfile("mypkg", "0.1")})
    c.run("export-pkg . -pr=myprofile")
    assert "conanfile.py (mypkg/0.1): Created package" in c.out


def test_export_pkg_tool_requires():
    """ when a package has "tool_requires" that it needs at the package() method, like
    typical cmake.install() or autotools.install() (tool_require msys2), then it is necessary:
    - to install the dependencies
    - to inject the tool-requirements
    - to propagate the environment and the conf
    """
    c = TestClient(default_server_user=True)
    tool = textwrap.dedent("""
        from conan import ConanFile
        class Tool(ConanFile):
            name = "tool"
            version = "0.1"
            def package_info(self):
                self.buildenv_info.define("MYVAR", "MYVALUE")
                self.conf_info.define("user.team:conf", "CONF_VALUE")
            """)
    consumer = textwrap.dedent("""
        import platform
        from conan import ConanFile

        class Consumer(ConanFile):
            name = "consumer"
            version = "0.1"
            tool_requires = "tool/0.1"
            def package(self):
                self.output.info(f"MYCONF {self.conf.get('user.team:conf')}")
                cmd = "set MYVAR" if platform.system() == "Windows" else "echo MYVAR=$MYVAR"
                self.run(cmd)
            """)
    c.save({"tool/conanfile.py": tool,
            "consumer/conanfile.py": consumer})

    c.run("create tool")
    c.run("export-pkg consumer")
    assert "conanfile.py (consumer/0.1): MYCONF CONF_VALUE" in c.out
    assert "MYVAR=MYVALUE" in c.out
    c.run("upload tool* -r=default -c")
    c.run("remove tool* -c")
    c.run("export-pkg consumer")
    assert "conanfile.py (consumer/0.1): MYCONF CONF_VALUE" in c.out
    assert "MYVAR=MYVALUE" in c.out


def test_export_pkg_output_folder():
    """ If the local build is using a different output-folder, it should work and export it
    """
    c = TestClient()
    consumer = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save, copy

        class Consumer(ConanFile):
            name = "consumer"
            version = "0.1"

            def build(self):
                save(self, "myfile.txt", "")
            def package(self):
                copy(self, "*", src=self.build_folder, dst=self.package_folder)
            """)
    c.save({"conanfile.py": consumer})

    c.run("build . -of=mytmp")
    c.run("export-pkg . -of=mytmp")
    assert "Packaged 1 '.txt' file: myfile.txt" in c.out
    assert os.path.exists(os.path.join(c.current_folder, "mytmp", "myfile.txt"))


def test_export_pkg_test_package():
    """ If there is a test_package, run it
    """
    c = TestClient()
    test_conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Test(ConanFile):
            def requirements(self):
                self.requires(self.tested_reference_str)
            def test(self):
                self.output.info("RUN TEST PACKAGE!!!!")
            """)
    c.save({"conanfile.py": GenConanfile("pkg", "1.0"),
            "test_package/conanfile.py": test_conanfile})

    c.run("export-pkg . ")
    assert "test_package" in c.out
    assert "RUN TEST PACKAGE!!!!" in c.out

    c.run('export-pkg . -tf=""')
    assert "test_package" not in c.out
    assert "RUN TEST" not in c.out


def test_export_pkg_test_package_build_require():
    """ Test --build-require
    """
    c = TestClient()
    test_conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Test(ConanFile):
            def build_requirements(self):
                self.tool_requires(self.tested_reference_str)
            def test(self):
                self.output.info(f"RUN TEST PACKAGE!!!!")
            """)
    c.save({"conanfile.py": GenConanfile("pkg", "1.0").with_setting("os"),
            "test_package/conanfile.py": test_conanfile})

    c.run("export-pkg . -s:b os=Windows -s:h os=Linux --build-require --lockfile-out=conan.lock")
    assert "test_package" in c.out
    assert "RUN TEST PACKAGE!!!!" in c.out
    lock = json.loads(c.load("conan.lock"))
    assert "pkg/1.0" in lock["build_requires"][0]


def test_export_pkg_remote_python_requires():
    """ Test that remote python-requires can be resolved
    """
    c = TestClient(default_server_user=True)
    c.save({"tool/conanfile.py": GenConanfile("tool", "1.0"),
            "pkg/conanfile.py": GenConanfile("pkg", "1.0").with_python_requires("tool/1.0")})

    c.run("create tool")
    c.run("upload tool* -r=default -c")
    c.run("remove * -c")
    c.run("export-pkg pkg")
    assert "conanfile.py (pkg/1.0): Exported package binary" in c.out


def test_remote_none():
    # https://github.com/conan-io/conan/pull/14705
    c = TestClient(default_server_user=True)
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
            "pkg/conanfile.py": GenConanfile("pkg", "0.1"),
            "pkg/test_package/conanfile.py": GenConanfile().with_test("pass").with_requires("dep/0.1")})
    c.run("create dep")
    c.run("upload dep* -r=default -c")
    c.run("build pkg")
    c.run("remove dep*:* -c")
    c.run("export-pkg pkg")    # This used to crash
    # No longer crash
    assert "pkg/0.1 (test package): Running test()" in c.out


def test_remote_none_tool_requires():
    # https://github.com/conan-io/conan/pull/14705
    c = TestClient(default_server_user=True)
    c.save({"tool/conanfile.py": GenConanfile("tool", "0.1").with_settings("compiler"),
            "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_tool_requires("tool/0.1"),
            "pkg/test_package/conanfile.py": GenConanfile().with_test("pass")})
    settings = "-s:b compiler=gcc -s:b compiler.version=9 -s:b compiler.libcxx=libstdc++11"
    c.run(f"create tool {settings} -s:b compiler.cppstd=20 --build-require")
    c.run(f"build pkg {settings} -s:b compiler.cppstd=17")
    c.run(f"export-pkg pkg {settings} -s:b compiler.cppstd=17")  # This used to crash
    # No longer crash
    assert "pkg/0.1 (test package): Running test()" in c.out


def test_export_pkg_set_version_python_requires():
    # https://github.com/conan-io/conan/issues/16223
    c = TestClient(light=True)
    py_require = textwrap.dedent("""
        from conan import ConanFile

        class TestBuild:
            def set_version(self):
                assert self.version

        class TestModule(ConanFile):
            name = "pyreq"
            version = "0.1"
            package_type = "python-require"
    """)
    pkg = textwrap.dedent("""
        from conan import ConanFile

        class TestPkgConan(ConanFile):
            name="testpkg"
            python_requires = "pyreq/0.1"
            python_requires_extend = "pyreq.TestBuild"
        """)
    c.save({"pyreq/conanfile.py": py_require,
            "pkg/conanfile.py": pkg})

    c.run("create pyreq")
    c.run("export-pkg pkg --version=1.0+0")
    assert "conanfile.py (testpkg/1.0+0): Exported package binary" in c.out



import textwrap
import os
import pytest
import docker
from conan.test.utils.tools import TestClient
from conan.test.assets.cmake import gen_cmakelists
from conan.test.assets.sources import gen_function_h, gen_function_cpp


def docker_from_env():
    try:
        return docker.from_env()
    except Exception:
        return docker.DockerClient(base_url=f'unix://{os.path.expanduser("~")}/.rd/docker.sock', version='auto') # Rancher


def docker_skip(test_image='ubuntu:22.04'):
    try:
        docker_client = docker_from_env()
        if test_image:
            docker_client.images.pull(test_image)
    except docker.errors.DockerException:
        return True
    except docker.errors.ImageNotFound:
        return True
    except docker.errors.APIError:
        return True
    return False


def conan_base_path():
    import conans
    return os.path.dirname(os.path.dirname(conans.__file__))


def dockerfile_path(name=None):
    path = os.path.join(os.path.dirname(__file__), "dockerfiles")
    if name:
        path = os.path.join(path, name)
    return path


@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_cache_shared():
    """
    Tests the ``conan create . ``
    """
    client = TestClient()
    profile_build = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    """)

    profile_host = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    dockerfile={dockerfile_path()}
    build_context={conan_base_path()}
    image=conan-runner-default-test
    cache=shared
    remove=True
    """)

    client.save({"host": profile_host, "build": profile_build})
    client.run("new cmake_lib -d name=pkg -d version=0.2")
    client.run("create . -pr:h host -pr:b build")

    assert "[100%] Built target example" in client.out
    assert "Removing container" in client.out


@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_cache_shared_profile_from_cache():
    """
    Tests the ``conan create . ``
    """
    client = TestClient()
    profile_build = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    """)

    profile_host = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    dockerfile={dockerfile_path()}
    build_context={conan_base_path()}
    image=conan-runner-default-test
    cache=shared
    remove=True
    """)

    client.save({"default_host": profile_host, "default_build": profile_build}, path=client.paths.profiles_path)
    client.run("new cmake_lib -d name=pkg -d version=0.2")
    client.run("create . -pr:h default_host -pr:b default_build")

    assert "[100%] Built target example" in client.out
    assert "Removing container" in client.out


@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_cache_shared_profile_folder():
    """
    Tests the ``conan create . ``
    """
    client = TestClient()
    profile_build = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    """)

    profile_host = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    dockerfile={dockerfile_path()}
    build_context={conan_base_path()}
    image=conan-runner-default-test
    cache=shared
    remove=True
    """)

    client.save({"build": profile_build})
    client.save({"docker_default": profile_host}, path = os.path.join(client.cache_folder, "profiles"))
    client.run("new cmake_lib -d name=pkg -d version=0.2")
    client.run("create . -pr:h docker_default -pr:b build")

    assert "[100%] Built target example" in client.out
    assert "Removing container" in client.out

@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_dockerfile_folder_path():
    """
    Tests the ``conan create . ``
    """
    client = TestClient()
    profile_build = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    """)

    profile_host_copy = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    dockerfile={dockerfile_path()}
    build_context={conan_base_path()}
    image=conan-runner-default-test
    cache=copy
    remove=True
    """)

    profile_host_clean = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    dockerfile={dockerfile_path()}
    build_context={conan_base_path()}
    image=conan-runner-default-test
    cache=clean
    remove=True
    """)

    client.save({"host_copy": profile_host_copy, "host_clean": profile_host_clean, "build": profile_build})
    client.run("new cmake_lib -d name=pkg -d version=0.2")
    client.run("create . -pr:h host_copy -pr:b build")

    assert "Restore: pkg/0.2" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe metadata" in client.out
    assert "Removing container" in client.out

    client.run("create . -pr:h host_clean -pr:b build")

    assert "Restore: pkg/0.2" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe metadata" in client.out
    assert "Removing container" in client.out


@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_profile_default_folder():
    """
    Tests the ``conan create . ``
    """
    client = TestClient()
    profile_build = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    """)
    profile_host = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    dockerfile={dockerfile_path("Dockerfile_test")}
    build_context={conan_base_path()}
    image=conan-runner-default-test
    cache=copy
    remove=True
    """)
    client.save({"host_from_profile": profile_host}, path = os.path.join(client.cache_folder, "profiles"))
    client.save({"build_from_profile": profile_build}, path = os.path.join(client.cache_folder, "profiles"))
    client.run("new cmake_lib -d name=pkg -d version=0.2")
    client.run("create . -pr:h host_from_profile -pr:b build_from_profile")

    assert "Container conan-runner-docker running" in client.out
    assert "Restore: pkg/0.2" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe metadata" in client.out
    assert "Removing container" in client.out


@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_dockerfile_file_path():
    """
    Tests the ``conan create . ``
    """
    client = TestClient()
    profile_build = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    """)
    profile_host = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    dockerfile={dockerfile_path("Dockerfile_test")}
    build_context={conan_base_path()}
    image=conan-runner-default-test
    cache=copy
    remove=True
    """)
    client.save({"host": profile_host, "build": profile_build})
    client.run("new cmake_lib -d name=pkg -d version=0.2")
    client.run("create . -pr:h host -pr:b build")

    assert "Container conan-runner-docker running" in client.out
    assert "Restore: pkg/0.2" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe metadata" in client.out
    assert "Removing container" in client.out


@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
@pytest.mark.parametrize("build_type,shared", [("Release", False), ("Debug", True)])
def test_create_docker_runner_with_ninja(build_type, shared):
    conanfile = textwrap.dedent("""
    import os
    from conan import ConanFile
    from conan.tools.cmake import CMake, CMakeToolchain

    class Library(ConanFile):
        name = "hello"
        version = "1.0"
        settings = 'os', 'arch', 'compiler', 'build_type'
        exports_sources = 'hello.h', '*.cpp', 'CMakeLists.txt'
        options = {'shared': [True, False]}
        default_options = {'shared': False}

        def generate(self):
            tc = CMakeToolchain(self, generator="Ninja")
            tc.generate()

        def build(self):
            cmake = CMake(self)
            cmake.configure()
            cmake.build()
            self.run(os.sep.join([".", "myapp"]))

        def package(self):
            cmake = CMake(self)
            cmake.install()
    """)

    client = TestClient(path_with_spaces=False)
    client.save({'conanfile.py': conanfile,
                "CMakeLists.txt": gen_cmakelists(libsources=["hello.cpp"],
                                                appsources=["main.cpp"],
                                                install=True),
                "hello.h": gen_function_h(name="hello"),
                "hello.cpp": gen_function_cpp(name="hello", includes=["hello"]),
                "main.cpp": gen_function_cpp(name="main", includes=["hello"],
                                            calls=["hello"])})
    profile = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    image=conan-runner-ninja-test
    dockerfile={dockerfile_path("Dockerfile_ninja")}
    build_context={conan_base_path()}
    cache=copy
    remove=True
    """)
    client.save({"profile": profile})
    settings = "-s os=Linux -s build_type={} -o hello/*:shared={}".format(build_type, shared)
    # create should also work
    client.run("create . --name=hello --version=1.0 {} -pr:h=profile -pr:b=profile".format(settings))
    assert 'cmake -G "Ninja"' in client.out
    assert "main: {}!".format(build_type) in client.out

@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_from_configfile():
    """
    Tests the ``conan create . ``
    """
    client = TestClient()
    configfile = textwrap.dedent(f"""
        image: conan-runner-default-test
        build:
            dockerfile: {dockerfile_path("Dockerfile_test")}
            build_context: {conan_base_path()}
        run:
            name: my-custom-conan-runner-container
        """)
    client.save({"configfile.yaml": configfile})


    profile_build = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    """)
    profile_host = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    configfile={os.path.join(client.current_folder, 'configfile.yaml')}
    cache=copy
    remove=True
    """)

    client.save({"host": profile_host, "build": profile_build})
    client.run("new cmake_lib -d name=pkg -d version=0.2")
    client.run("create . -pr:h 'host' -pr:b 'build'")

    assert "Container my-custom-conan-runner-container running" in client.out
    assert "Restore: pkg/0.2" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe metadata" in client.out
    assert "Removing container" in client.out


@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_from_configfile_with_args():
    """
    Tests the ``conan create . ``
    """
    client = TestClient()

    # Ensure the network exists
    docker_client = docker_from_env()
    docker_client.networks.create("my-network")

    configfile = textwrap.dedent(f"""
        image: conan-runner-default-test-with-args
        build:
            dockerfile: {dockerfile_path("Dockerfile_args")}
            build_context: {conan_base_path()}
            build_args:
                BASE_IMAGE: ubuntu:22.04
        run:
            name: my-conan-runner-container-with-args
            network: my-network
        """)
    client.save({"configfile.yaml": configfile})


    profile_build = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    """)
    profile_host = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    configfile={os.path.join(client.current_folder, 'configfile.yaml')}
    cache=copy
    remove=True
    """)

    client.save({"host": profile_host, "build": profile_build})
    client.run("new cmake_lib -d name=pkg -d version=0.2")
    client.run("create . -pr:h 'host' -pr:b 'build'")

    assert "command/dockerfiles/Dockerfile_args" in client.out
    assert "Restore: pkg/0.2" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe metadata" in client.out
    assert "Removing container" in client.out

    docker_client.networks.get("my-network").remove()

@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_default_build_profile():
    """
    Tests the ``conan create . ``
    """
    client = TestClient()

    profile_host = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    dockerfile={dockerfile_path()}
    build_context={conan_base_path()}
    image=conan-runner-default-test
    cache=clean
    remove=True
    """)

    client.save({"host_clean": profile_host})
    client.run("new cmake_lib -d name=pkg -d version=0.2")
    client.run("create . -pr:h host_clean -vverbose")

    assert "Copying default profile" in client.out
    assert "Restore: pkg/0.2" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe" in client.out
    assert "Restore: pkg/0.2:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe metadata" in client.out
    assert "Removing container" in client.out

@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip('ubuntu:22.04'), reason="Only docker running")
def test_create_docker_runner_profile_composition():
    """
    Tests the ``conan create . `` with profile composition
    """
    client = TestClient()

    profile = textwrap.dedent(f"""\
    [settings]
    arch={{{{ detect_api.detect_arch() }}}}
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [runner]
    type=docker
    image=conan-runner-ninja-test
    """)

    profile_extension = textwrap.dedent(f"""\
    [runner]
    type=docker
    dockerfile={dockerfile_path("Dockerfile_ninja")}
    build_context={conan_base_path()}
    cache=copy
    remove=True
    """)
    client.save({"profile": profile, "profile_extension": profile_extension})
    client.run("new cmake_lib -d name=pkg -d version=2.0")
    client.run("create . -pr:h profile -pr:h profile_extension")

    assert "[100%] Built target example" in client.out
    assert "Restore: pkg/2.0 in pkgc6abef0178849" in client.out
    assert "Restore: pkg/2.0:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe" in client.out
    assert "Restore: pkg/2.0:8631cf963dbbb4d7a378a64a6fd1dc57558bc2fe metadata" in client.out


@pytest.mark.docker_runner
@pytest.mark.skipif(docker_skip(), reason="Only docker running")
def test_create_docker_runner_in_subfolder():
    client = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import load, copy
        from conan.tools.cmake import CMake

        class Pkg(ConanFile):
            name = "pkg"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"

            def layout(self):
                self.folders.root = ".."
                self.folders.source = "."
                self.folders.build = "build"

            def export_sources(self):
                folder = os.path.join(self.recipe_folder, "..")
                copy(self, "*.txt", folder, self.export_sources_folder)
                copy(self, "src/*.cpp", folder, self.export_sources_folder)
                copy(self, "include/*.h", folder, self.export_sources_folder)

            def source(self):
                cmake_file = load(self, "CMakeLists.txt")

            def build(self):
                path = os.path.join(self.source_folder, "CMakeLists.txt")
                cmake_file = load(self, path)
                cmake = CMake(self)
                cmake.configure()
                cmake.build()

            def package(self):
                cmake = CMake(self)
                cmake.install()
            """)

    header = textwrap.dedent("""
        #pragma once
        void hello();
        """)
    source = textwrap.dedent("""
        #include <iostream>
        void hello() {
            std::cout << "Hello!" << std::endl;
        }
        """)

    cmakelist = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(pkg CXX)
        add_library(pkg src/hello.cpp)
        target_include_directories(pkg PUBLIC include)
        set_target_properties(pkg PROPERTIES PUBLIC_HEADER "include/hello.h")
        install(TARGETS pkg)

        """)

    profile_host = textwrap.dedent(f"""\
        [settings]
        arch={{{{ detect_api.detect_arch() }}}}
        build_type=Release
        compiler=gcc
        compiler.cppstd=gnu17
        compiler.libcxx=libstdc++11
        compiler.version=11
        os=Linux
        [runner]
        type=docker
        dockerfile={dockerfile_path()}
        build_context={conan_base_path()}
        image=conan-runner-default-test
        cache=clean
        remove=True
        """)

    client.save({"conan/conanfile.py": conanfile,
                "conan/host": profile_host,
                "include/hello.h": header,
                "src/hello.cpp": source,
                "CMakeLists.txt": cmakelist})

    with client.chdir("conan"):
        client.run("create . -pr:h host -vverbose")

    assert "Restore: pkg/1.0" in client.out
    assert "Removing container" in client.out



import json
import os

import pytest
from requests import Response

from requests.exceptions import ConnectionError

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient, TestRequester, TestServer
from conan.internal.util.files import save


class TestBrokenDownload:

    @pytest.fixture()
    def setup(self):
        server = TestServer()
        client = TestClient(servers={"default": server}, inputs=["admin", "password"])
        client.save({"conanfile.py": GenConanfile("hello", "0.1")})
        client.run("create .")
        pref = client.created_package_reference("hello/0.1")
        client.run("upload * -r default -c")
        client.run("remove * -c")
        return client, pref

    def test_corrupt_export_tgz(self, setup):
        client, pref = setup
        server = client.servers["default"]
        path = server.test_server.server_store.export(pref.ref)
        tgz = os.path.join(path, "conan_export.tgz")
        save(tgz, "contents")  # dummy content to break it, so the download decompress will fail
        client.run("install --requires=hello/0.1", assert_error=True)
        assert "Error while extracting downloaded file" in client.out
        assert not os.path.exists(client.get_latest_ref_layout(pref.ref).export())

    def test_remove_conaninfo(self, setup):
        """
        if the conaninfo is removed, it is considered at least a broken package by the client
        """
        client, pref = setup
        server = client.servers["default"]
        path = server.test_server.server_store.package(pref)
        conaninfo = os.path.join(path, "conaninfo.txt")
        os.unlink(conaninfo)
        client.run("install --requires=hello/0.1", assert_error=True)
        assert "ERROR: Corrupted hello/0.1:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
               " in 'default' remote: no conaninfo.txt" in client.out

    def test_remove_conanfile(self, setup):
        """
        if the conanfile is removed, it is considered at least a broken package by the client
        """
        client, pref = setup
        server = client.servers["default"]
        path = server.test_server.server_store.export(pref.ref)
        conanfile = os.path.join(path, "conanfile.py")
        os.unlink(conanfile)
        client.run("install --requires=hello/0.1", assert_error=True)
        assert "Corrupted hello/0.1 in 'default' remote: no conanfile.py" in client.out

    def test_remove_pkg_conanmanifest(self, setup):
        """
        if the manifest is missing the server side can say there is no package
        """
        client, pref = setup
        server = client.servers["default"]

        path = server.test_server.server_store.package(pref)
        manifest = os.path.join(path, "conanmanifest.txt")
        os.unlink(manifest)
        client.run("install --requires=hello/0.1", assert_error=True)
        assert "ERROR: Binary package not found: 'hello/0.1" in client.out

    def test_remove_recipe_conanmanifest(self, setup):
        """
        if the manifest is missing the server side can say there is no recipe
        """
        client, pref = setup
        server = client.servers["default"]

        path = server.test_server.server_store.export(pref.ref)
        manifest = os.path.join(path, "conanmanifest.txt")
        os.unlink(manifest)
        client.run("install --requires=hello/0.1", assert_error=True)
        assert "Recipe not found: 'hello/0.1" in client.out


def test_client_retries():
    server = TestServer()
    servers = {"default": server}
    client = TestClient(servers=servers, inputs=["admin", "password"])
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=lib --version=1.0 --user=lasote --channel=stable")
    client.run("upload lib/1.0@lasote/stable -c -r default")

    class DownloadFilesBrokenRequester(TestRequester):
        def __init__(self, times_to_fail=1, *args, **kwargs):
            self.times_to_fail = times_to_fail
            super(DownloadFilesBrokenRequester, self).__init__(*args, **kwargs)

        def get(self, url, **kwargs):
            # conaninfo is skipped sometimes from the output, use manifest
            if "conanmanifest.txt" in url and self.times_to_fail > 0:
                self.times_to_fail = self.times_to_fail - 1
                raise ConnectionError("Fake connection error exception")
            else:
                return super(DownloadFilesBrokenRequester, self).get(url, **kwargs)

    def DownloadFilesBrokenRequesterTimesOne(*args, **kwargs):  # noqa
        return DownloadFilesBrokenRequester(1, *args, **kwargs)
    client = TestClient(servers=servers, inputs=["admin", "password"],
                        requester_class=DownloadFilesBrokenRequesterTimesOne)
    client.run("install --requires=lib/1.0@lasote/stable")
    assert "WARN: network: Error downloading file" in client.out
    assert 'Fake connection error exception' in client.out
    assert 1 == str(client.out).count("Waiting 0 seconds to retry...")

    client = TestClient(servers=servers, inputs=["admin", "password"],
                        requester_class=DownloadFilesBrokenRequesterTimesOne)
    client.save_home({"global.conf": "core.download:retry_wait=1"})
    client.run("install --requires=lib/1.0@lasote/stable")
    assert 1 == str(client.out).count("Waiting 1 seconds to retry...")

    def DownloadFilesBrokenRequesterTimesTen(*args, **kwargs):  # noqa
        return DownloadFilesBrokenRequester(10, *args, **kwargs)
    client = TestClient(servers=servers, inputs=["admin", "password"],
                        requester_class=DownloadFilesBrokenRequesterTimesTen)
    client.save_home({"global.conf": "core.download:retry_wait=0\n"
                                     "core.download:retry=11"})
    client.run("install --requires=lib/1.0@lasote/stable")
    assert 10 == str(client.out).count("Waiting 0 seconds to retry...")


def test_forbidden_blocked_conanmanifest():
    """ this is what happens when a server blocks downloading a specific file
    """
    server = TestServer()
    servers = {"default": server}
    client = TestClient(servers=servers, inputs=["admin", "password"])
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=lib --version=1.0")
    client.run("upload lib/1.0* -c -r default")

    class DownloadForbidden(TestRequester):
        def get(self, url, **kwargs):
            if "conanmanifest.txt" in url:
                r = Response()
                r._content = "Forbidden because of security!!!"
                r.status_code = 403
                return r
            else:
                return super(DownloadForbidden, self).get(url, **kwargs)

    client = TestClient(servers=servers, inputs=["admin", "password"],
                        requester_class=DownloadForbidden)
    client.run("download lib/1.0 -r=default", assert_error=True)
    assert "Forbidden because of security!!!" in client.out

    client.run("list *")
    assert "lib/1.0" not in client.out

    client.run("install --requires=lib/1.0", assert_error=True)
    assert "Forbidden because of security!!!" in client.out

    client.run("list *")
    assert "lib/1.0" not in client.out


def test_forbidden_blocked_package_conanmanifest():
    """ this is what happens when a server blocks downloading a specific file
    """
    server = TestServer()
    servers = {"default": server}
    client = TestClient(servers=servers, inputs=["admin", "password"])
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=lib --version=1.0")
    client.run("upload lib/1.0* -c -r default")

    class DownloadForbidden(TestRequester):
        def get(self, url, **kwargs):
            if "packages/" in url and "conanmanifest.txt" in url:
                r = Response()
                r._content = "Forbidden because of security!!!"
                r.status_code = 403
                return r
            else:
                return super(DownloadForbidden, self).get(url, **kwargs)

    client = TestClient(servers=servers, inputs=["admin", "password"],
                        requester_class=DownloadForbidden)
    client.run("download lib/1.0 -r=default", assert_error=True)

    def check_cache():
        assert "Forbidden because of security!!!" in client.out
        client.run("list *:* --format=json")
        listjson = json.loads(client.stdout)
        revisions = listjson["Local Cache"]["lib/1.0"]["revisions"]
        packages = revisions["4d670581ccb765839f2239cc8dff8fbd"]["packages"]
        assert packages == {}  # No binaries"

    check_cache()

    client.run("install --requires=lib/1.0", assert_error=True)
    assert "Forbidden because of security!!!" in client.out
    check_cache()



import json
import os
import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.fixture()
def client():
    c = TestClient(default_server_user=True)
    dep = textwrap.dedent("""
        from conan import ConanFile
        class Dep(ConanFile):
            name = "dep"
            version = "0.1"
            def source(self):
                self.output.info("RUNNING SOURCE!!")
            """)

    c.save({"dep/conanfile.py": dep})
    c.run("create dep")
    c.run("upload * -c -r=default")
    c.run("remove * -c")
    return c


def test_install(client):
    client.run("install --requires=dep/0.1")
    assert "RUNNING SOURCE" not in client.out
    client.run("install --requires=dep/0.1 -c tools.build:download_source=True --format=json")
    assert "RUNNING SOURCE" in client.out
    graph = json.loads(client.stdout)
    zlib = graph["graph"]["nodes"]["1"]
    assert os.path.exists(zlib["source_folder"])
    client.run("install --requires=dep/0.1 -c tools.build:download_source=True --format=json")
    assert "RUNNING SOURCE" not in client.out
    graph = json.loads(client.stdout)
    zlib = graph["graph"]["nodes"]["1"]
    assert os.path.exists(zlib["source_folder"])


def test_info(client):
    client.run("graph info --requires=dep/0.1")
    assert "RUNNING SOURCE" not in client.out
    # Even if the package is to be built, it shouldn't download sources unless the conf is defined
    client.run("graph info --requires=dep/0.1 --build=dep*")
    assert "RUNNING SOURCE" not in client.out
    client.run("graph info --requires=dep/0.1 -c tools.build:download_source=True")
    assert "RUNNING SOURCE" in client.out
    client.run("graph info --requires=dep/0.1 -c tools.build:download_source=True")
    assert "RUNNING SOURCE" not in client.out


def test_info_editable():
    """ graph info for editable shouldn't crash, but it also shoudn't do anythin
    # https://github.com/conan-io/conan/issues/15003
    """
    c = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile

        class Dep(ConanFile):
            name = "dep"
            version = "0.1"

            def source(self):
                self.output.info("RUNNING SOURCE!!")
            """)

    c.save({"conanfile.py": dep})
    c.run("editable add .")
    c.run("graph info --requires=dep/0.1")
    assert "RUNNING SOURCE" not in c.out
    c.run("graph info --requires=dep/0.1 -c tools.build:download_source=True")
    assert "RUNNING SOURCE" not in c.out  # BUT it doesn't crash, it used to crash



import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_package_vendor():
    c = TestClient()
    app = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import copy, save

        class App(ConanFile):
            name = "app"
            version = "0.1"
            package_type = "application"
            vendor = True
            requires = "pkga/0.1"
            def package(self):
                copy(self, "*", src=self.dependencies["pkga"].package_folder,
                     dst=self.package_folder)
                save(self, os.path.join(self.package_folder, "app.exe"), "app")
            """)

    c.save({"pkga/conanfile.py": GenConanfile("pkga", "0.1").with_package_type("shared-library")
                                                            .with_package_file("pkga.dll", "dll"),
            "app/conanfile.py": app
            })
    c.run("create pkga")
    c.run("create app")  # -c tools.graph:vendor=build will be automatic
    assert "app/0.1: package(): Packaged 1 '.dll' file: pkga.dll" in c.out

    # we can safely remove pkga
    c.run("remove pkg* -c")
    c.run("list app:*")
    assert "pkga" not in c.out  # The binary doesn't depend on pkga
    c.run("install --requires=app/0.1 --deployer=full_deploy")
    assert "pkga" not in c.out
    assert c.load("full_deploy/host/app/0.1/app.exe") == "app"
    assert c.load("full_deploy/host/app/0.1/pkga.dll") == "dll"

    # we can create a modified pkga
    c.save({"pkga/conanfile.py": GenConanfile("pkga", "0.1").with_package_type("shared-library")
           .with_package_file("pkga.dll", "newdll")})
    c.run("create pkga")
    # still using the re-packaged one
    c.run("install --requires=app/0.1 --deployer=full_deploy")
    assert "pkga" not in c.out
    assert c.load("full_deploy/host/app/0.1/app.exe") == "app"
    assert c.load("full_deploy/host/app/0.1/pkga.dll") == "dll"

    # but we can force the expansion, still not the rebuild
    c.run("install --requires=app/0.1 --deployer=full_deploy -c tools.graph:vendor=build")
    assert "pkga" in c.out
    assert c.load("full_deploy/host/app/0.1/app.exe") == "app"
    assert c.load("full_deploy/host/app/0.1/pkga.dll") == "dll"

    # and finally we can force the expansion and the rebuild
    c.run("install --requires=app/0.1 --build=app* --deployer=full_deploy "
          "-c tools.graph:vendor=build")
    assert "pkga" in c.out
    assert c.load("full_deploy/host/app/0.1/app.exe") == "app"
    assert c.load("full_deploy/host/app/0.1/pkga.dll") == "newdll"
    # This shoulnd't happen, no visibility over transitive dependencies of app
    assert not os.path.exists(os.path.join(c.current_folder, "full_deploy", "host", "pkga"))

    # lets remove the binary
    c.run("remove app:* -c")
    c.run("install --requires=app/0.1", assert_error=True)
    assert "Missing binary" in c.out
    c.run("install --requires=app/0.1 --build=missing", assert_error=True)
    assert "app/0.1: Invalid: The package 'app/0.1' is a vendoring one, needs to be built " \
           "from source, but it didn't enable 'tools.graph:vendor=build'" in c.out

    c.run("install --requires=app/0.1 --build=missing  -c tools.graph:vendor=build")
    assert "pkga" in c.out  # it works


def test_package_vendor_editable():
    c = TestClient()
    pkgb = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import copy, save

        class App(ConanFile):
            name = "pkgb"
            version = "0.1"
            package_type = "shared-library"
            vendor = True
            requires = "pkga/0.1"
            def layout(self):
                self.folders.build = "build"
                self.cpp.build.bindirs = ["build"]
            def generate(self):
                copy(self, "*", src=self.dependencies["pkga"].package_folder,
                     dst=self.build_folder)
            def build(self):
                save(self, os.path.join(self.build_folder, "pkgb.dll"), "dll")
            """)

    c.save({"pkga/conanfile.py": GenConanfile("pkga", "0.1").with_package_type("shared-library")
                                                            .with_package_file("bin/pkga.dll", "d"),
            "pkgb/conanfile.py": pkgb,
            "app/conanfile.py": GenConanfile("app", "0.1").with_settings("os")
                                                          .with_requires("pkgb/0.1")
            })
    c.run("create pkga")
    c.run("editable add pkgb")
    c.run("install app -s os=Linux --build=editable")
    assert "pkga" in c.out
    # The environment file of "app" doesn't have any visibility of the "pkga" paths
    envfile_app = c.load("app/conanrunenv.sh")
    assert "pkga" not in envfile_app
    # But the environment file needed to build "pkgb" has visibility over the "pkga" paths
    envfile_pkgb = c.load("pkgb/conanrunenv.sh")
    assert "pkga" in envfile_pkgb


def test_vendor_dont_propagate_options():
    c = TestClient()
    app = GenConanfile("app", "0.1").with_requires("pkga/0.1").with_class_attribute("vendor=True")
    c.save({"pkga/conanfile.py": GenConanfile("pkga", "0.1").with_shared_option(False),
            "app/conanfile.py": app,
            "consumer/conanfile.txt": "[requires]\napp/0.1",
            "consumer_shared/conanfile.txt": "[requires]\napp/0.1\n[options]\n*:shared=True"
            })
    c.run("create pkga")
    c.assert_listed_binary({"pkga/0.1": ("55c609fe8808aa5308134cb5989d23d3caffccf2", "Build")})
    c.run("create app")
    c.assert_listed_binary({"pkga/0.1": ("55c609fe8808aa5308134cb5989d23d3caffccf2", "Cache"),
                            "app/0.1": ("da39a3ee5e6b4b0d3255bfef95601890afd80709", "Build")})
    c.run("install consumer --build=app/* -c tools.graph:vendor=build")
    c.assert_listed_binary({"pkga/0.1": ("55c609fe8808aa5308134cb5989d23d3caffccf2", "Cache"),
                            "app/0.1": ("da39a3ee5e6b4b0d3255bfef95601890afd80709", "Build")})
    c.run("install consumer_shared --build=app/* -c tools.graph:vendor=build")
    c.assert_listed_binary({"pkga/0.1": ("55c609fe8808aa5308134cb5989d23d3caffccf2", "Cache"),
                            "app/0.1": ("da39a3ee5e6b4b0d3255bfef95601890afd80709", "Build")})


def test_package_vendor_export_pkg():
    c = TestClient()
    app = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import copy, save

        class App(ConanFile):
            name = "app"
            version = "0.1"
            package_type = "application"
            vendor = True
            requires = "pkga/0.1"
            def package(self):
                copy(self, "*", src=self.dependencies["pkga"].package_folder,
                     dst=self.package_folder)
                save(self, os.path.join(self.package_folder, "app.exe"), "app")
            """)

    c.save({"pkga/conanfile.py": GenConanfile("pkga", "0.1").with_package_type("shared-library")
                                                            .with_package_file("pkga.dll", "dll"),
            "app/conanfile.py": app
            })
    c.run("create pkga")
    c.run("build app")  # -c tools.graph:vendor=build will be automatic
    c.run("export-pkg app")
    assert "pkga/0.1" in c.out  # In the export-pkg process, dependencies are still needed
    assert "conanfile.py (app/0.1): package(): Packaged 1 '.dll' file: pkga.dll" in c.out

    # we can safely remove pkga, once the package is created, it can work without deps
    c.run("remove pkg* -c")
    c.run("list app:*")
    assert "pkga" not in c.out  # The binary doesn't depend on pkga
    c.run("install --requires=app/0.1 --deployer=full_deploy")
    assert "pkga" not in c.out
    assert c.load("full_deploy/host/app/0.1/app.exe") == "app"
    assert c.load("full_deploy/host/app/0.1/pkga.dll") == "dll"



from conan.internal.rest.client_routes import ClientV2Router
from conan.internal.util.files import sha1sum


def add_urls(package_list, remote):
    router = ClientV2Router(remote.url.rstrip("/"))
    for ref, packages in package_list.items():
        ref_info = package_list.recipe_dict(ref)
        for f, fp in ref_info.get("files", {}).items():
            ref_info.setdefault("upload-urls", {})[f] = {
                'url': router.recipe_file(ref, f), 'checksum': sha1sum(fp)
            }
        for pref in packages:
            pref_info = package_list.package_dict(pref)
            for f, fp in pref_info.get("files", {}).items():
                pref_info.setdefault("upload-urls", {})[f] = {
                    'url': router.package_file(pref, f), 'checksum': sha1sum(fp)
                }



import os
import platform
import textwrap

import pytest

from conan.tools.apple.apple import _to_apple_arch, XCRun
from conan.test.assets.sources import gen_function_cpp, gen_function_h
from conan.test.utils.mocks import ConanFileMock
from conan.test.utils.tools import TestClient
from conan.internal.util.runners import conan_run


@pytest.mark.tool("meson")
@pytest.mark.skipif(platform.system() != "Darwin", reason="requires OSX")
def test_apple_meson_toolchain_cross_compiling():
    arch_host = 'armv8' if platform.machine() == "x86_64" else "x86_64"
    arch_build = 'armv8' if platform.machine() != "x86_64" else "x86_64"
    profile = textwrap.dedent(f"""
    [settings]
    os = Macos
    arch = {arch_host}
    compiler = apple-clang
    compiler.version = 13.0
    compiler.libcxx = libc++
    """)
    profile_build = textwrap.dedent(f"""
    [settings]
    os = Macos
    arch = {arch_build}
    compiler = apple-clang
    compiler.version = 13.0
    compiler.libcxx = libc++
    """)
    conanfile_py = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import Meson, MesonToolchain
    from conan.tools.build import cross_building

    class App(ConanFile):
        settings = "os", "arch", "compiler", "build_type"
        options = {"shared": [True, False], "fPIC": [True, False]}
        default_options = {"shared": False, "fPIC": True}

        def layout(self):
            self.folders.build = "build"

        def config_options(self):
            if self.settings.os == "Windows":
                self.options.rm_safe("fPIC")

        def configure(self):
            if self.options.shared:
                self.options.rm_safe("fPIC")

        def generate(self):
            tc = MesonToolchain(self)
            tc.generate()
            # Forcing to create the native context too
            if cross_building(self):
                tc = MesonToolchain(self, native=True)
                tc.generate()

        def build(self):
            meson = Meson(self)
            meson.configure()
            meson.build()
    """)
    meson_build = textwrap.dedent("""
    project('tutorial', 'cpp')
    hello = library('hello', 'hello.cpp')
    # Even cross-building the library, we want to create an executable using only the native context
    executable('mygen', 'mygen.cpp', native: true)
    """)
    my_gen_cpp = gen_function_cpp(name="main")
    hello_h = gen_function_h(name="hello")
    hello_cpp = gen_function_cpp(name="hello")
    client = TestClient()
    client.save({"conanfile.py": conanfile_py,
                 "meson.build": meson_build,
                 "hello.h": hello_h,
                 "hello.cpp": hello_cpp,
                 "mygen.cpp": my_gen_cpp,
                 "profile_host": profile,
                 "profile_build": profile_build})
    client.run("build . --profile:build=profile_build --profile:host=profile_host")
    libhello = os.path.join(client.current_folder, "build", "libhello.a")
    assert os.path.isfile(libhello) is True
    # Now, ensuring that we can run the mygen executable
    mygen = os.path.join(client.current_folder, "build", "mygen")
    client.run_command(f"'{mygen}'")
    assert "Release!" in client.out
    # Extra check for lib arch
    conanfile = ConanFileMock({}, runner=conan_run)
    xcrun = XCRun(conanfile)
    lipo = xcrun.find('lipo')
    client.run_command('"%s" -info "%s"' % (lipo, libhello))
    assert "architecture: %s" % _to_apple_arch(arch_host) in client.out



import os

from conan.api.conan_api import ConanAPI
from conan.internal.cache.home_paths import HomePaths
from conan.internal.util.files import save
from conan.test.utils.test_files import temp_folder


def test_requester_reinit():
    """ Ensure the reinitialization of the requester gets the new global configuration """
    tmp_folder = temp_folder()
    home_path = HomePaths(tmp_folder)
    save(os.path.join(home_path.profiles_path, "default"), "")
    conan_api = ConanAPI(tmp_folder)
    save(os.path.join(home_path.global_conf_path), "core.net.http:timeout=(10, 20)")
    conan_api.reinit()
    assert conan_api._api_helpers.requester._timeout == (10, 20)



import pytest

from conan.test.utils.tools import TestClient

test_conanfile = """from conan import ConanFile

class Test(ConanFile):
    options = {"shared": [True, False]}
    default_options = {"shared": False}

    def requirements(self):
        self.requires(self.tested_reference_str)
        self.output.info("shared (requirements): %s" % (self.options.shared))

    def configure(self):
        self.output.info("shared (configure): %s" % (self.options.shared))

    def build(self):
        self.output.info("shared (build): %s" % (self.options.shared))

    def test(self):
        self.output.info("shared (test): %s" % (self.options.shared))
"""


dep = """from conan import ConanFile

class PkgConan(ConanFile):
    name = "dep"
    version = "0.1"
    options = {"shared": [True, False]}
    default_options = {"shared": False}

    def configure(self):
        self.output.info("shared (configure): %s" % str(self.options.shared))

    def requirements(self):
        self.output.info("shared (requirements): %s" % str(self.options.shared))

    def build(self):
        self.output.info("shared (build): %s" % str(self.options.shared))
"""


conanfile = """from conan import ConanFile

class PkgConan(ConanFile):
    name = "pkg"
    version = "0.1"
    options = {"shared": [True, False]}
    default_options = {"shared": False}

    def configure(self):
        self.output.info("shared (configure): %s" % str(self.options.shared))

    def requirements(self):
        self.requires("dep/0.1")
        self.output.info("shared (requirements): %s" % str(self.options.shared))

    def build(self):
        self.output.info("shared (build): %s" % str(self.options.shared))
"""


class TestPackageOptionsCreate:

    def test_test_package(self):
        """ non scoped options will be put in the package scope
        -o shared=True <=> -o pkg:shared=True
        """
        c = TestClient()
        c.save({"dep/conanfile.py": dep,
                "conanfile.py": conanfile,
                "test_package/conanfile.py": test_conanfile})
        c.run("export dep")
        c.run("create . -o shared=True --build=missing")

        assert "dep/0.1: shared (configure): False" in c.out
        assert "dep/0.1: shared (requirements): False" in c.out
        assert "dep/0.1: shared (build): False" in c.out

        assert "pkg/0.1: shared (configure): True" in c.out
        assert "pkg/0.1: shared (requirements): True" in c.out
        assert "pkg/0.1: shared (build): True" in c.out

        assert "pkg/0.1 (test package): shared (configure): False" in c.out
        assert "pkg/0.1 (test package): shared (requirements): False" in c.out
        assert "pkg/0.1 (test package): shared (build): False" in c.out
        assert "pkg/0.1 (test package): shared (test): False" in c.out

    def test_test_package_all_shared(self):
        """
        all shared should affect both the package and the test_package
        """
        c = TestClient()
        c.save({"dep/conanfile.py": dep,
                "conanfile.py": conanfile,
                "test_package/conanfile.py": test_conanfile})
        c.run("export dep")
        c.run("create . -o *:shared=True --build=missing")

        assert "dep/0.1: shared (configure): True" in c.out
        assert "dep/0.1: shared (requirements): True" in c.out
        assert "dep/0.1: shared (build): True" in c.out

        assert "pkg/0.1: shared (configure): True" in c.out
        assert "pkg/0.1: shared (requirements): True" in c.out
        assert "pkg/0.1: shared (build): True" in c.out

        assert "pkg/0.1 (test package): shared (configure): True" in c.out
        assert "pkg/0.1 (test package): shared (requirements): True" in c.out
        assert "pkg/0.1 (test package): shared (build): True" in c.out
        assert "pkg/0.1 (test package): shared (test): True" in c.out

    def test_test_package_consumers(self):
        c = TestClient()
        c.save({"dep/conanfile.py": dep,
                "conanfile.py": conanfile,
                "test_package/conanfile.py": test_conanfile})
        c.run("export dep")
        c.run("create . -o &:shared=True --build=missing")

        assert "dep/0.1: shared (configure): False" in c.out
        assert "dep/0.1: shared (requirements): False" in c.out
        assert "dep/0.1: shared (build): False" in c.out

        assert "pkg/0.1: shared (configure): True" in c.out
        assert "pkg/0.1: shared (requirements): True" in c.out
        assert "pkg/0.1: shared (build): True" in c.out

        assert "pkg/0.1 (test package): shared (configure): True" in c.out
        assert "pkg/0.1 (test package): shared (requirements): True" in c.out
        assert "pkg/0.1 (test package): shared (build): True" in c.out
        assert "pkg/0.1 (test package): shared (test): True" in c.out

        c.run("install --requires=dep/0.1 -o &:shared=True -b=missing")
        assert "dep/0.1: shared (configure): True" in c.out
        assert "dep/0.1: shared (requirements): True" in c.out
        assert "dep/0.1: shared (build): True" in c.out

    def test_test_package_non_consumers(self):
        c = TestClient()
        c.save({"dep/conanfile.py": dep,
                "conanfile.py": conanfile,
                "test_package/conanfile.py": test_conanfile})
        c.run("export dep")
        c.run("create . -o !&:shared=True --build=missing")

        assert "dep/0.1: shared (configure): True" in c.out
        assert "dep/0.1: shared (requirements): True" in c.out
        assert "dep/0.1: shared (build): True" in c.out

        assert "pkg/0.1: shared (configure): False" in c.out
        assert "pkg/0.1: shared (requirements): False" in c.out
        assert "pkg/0.1: shared (build): False" in c.out

        assert "pkg/0.1 (test package): shared (configure): False" in c.out
        assert "pkg/0.1 (test package): shared (requirements): False" in c.out
        assert "pkg/0.1 (test package): shared (build): False" in c.out
        assert "pkg/0.1 (test package): shared (test): False" in c.out

    def test_test_package_only(self):
        c = TestClient()
        c.save({"dep/conanfile.py": dep,
                "conanfile.py": conanfile,
                "test_package/conanfile.py": test_conanfile})
        c.run("export dep")
        c.run("create . -o &:shared=True -o shared=False --build=missing")

        assert "dep/0.1: shared (configure): False" in c.out
        assert "dep/0.1: shared (requirements): False" in c.out
        assert "dep/0.1: shared (build): False" in c.out

        assert "pkg/0.1: shared (configure): False" in c.out
        assert "pkg/0.1: shared (requirements): False" in c.out
        assert "pkg/0.1: shared (build): False" in c.out

        assert "pkg/0.1 (test package): shared (configure): True" in c.out
        assert "pkg/0.1 (test package): shared (requirements): True" in c.out
        assert "pkg/0.1 (test package): shared (build): True" in c.out
        assert "pkg/0.1 (test package): shared (test): True" in c.out


class TestPackageOptionsInstall:

    @pytest.mark.parametrize("pattern", ["", "*:", "&:"])
    def test_test_package(self, pattern):
        c = TestClient()
        c.save({"dep/conanfile.py": dep,
                "conanfile.py": conanfile})
        c.run("export dep")
        c.run(f"build . -o {pattern}shared=True --build=missing")

        dep_shared = "False" if "*" not in pattern else "True"
        assert f"dep/0.1: shared (configure): {dep_shared}" in c.out
        assert f"dep/0.1: shared (requirements): {dep_shared}" in c.out
        assert f"dep/0.1: shared (build): {dep_shared}" in c.out

        assert "conanfile.py (pkg/0.1): shared (configure): True" in c.out
        assert "conanfile.py (pkg/0.1): shared (requirements): True" in c.out
        assert "conanfile.py (pkg/0.1): shared (build): True" in c.out



import platform
import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
@pytest.mark.parametrize(
    "compiler, version",
    [
        ("msvc", "190"),
        ("msvc", "191"),
        ("clang", "16")
    ],
)
@pytest.mark.parametrize("runtime", ["dynamic", "static"])
@pytest.mark.parametrize("runtime_type", ["Release", "Debug"])
def test_toolchain_win(compiler, version, runtime, runtime_type):
    client = TestClient(path_with_spaces=False)
    settings = {"compiler": compiler,
                "compiler.version": version,
                "compiler.cppstd": "14",
                "compiler.runtime": runtime,
                "compiler.runtime_type": runtime_type,
                "build_type": "Release",
                "arch": "x86_64"}
    if compiler == "clang":
        settings["compiler.runtime_version"] = "v144"

    # Build the profile according to the settings provided
    settings = " ".join('-s %s="%s"' % (k, v) for k, v in settings.items() if v)

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.microsoft import MSBuildToolchain
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def generate(self):
                msbuild = MSBuildToolchain(self)
                msbuild.properties["IncludeExternals"] = "true"
                msbuild.generate()
            """)
    client.save({"conanfile.py": conanfile})
    client.run(f"install . {settings} -c tools.microsoft.msbuild:installation_path=")
    props = client.load("conantoolchain_release_x64.props")
    assert "<IncludeExternals>true</IncludeExternals>" in props
    assert "<LanguageStandard>stdcpp14</LanguageStandard>" in props
    if compiler == "msvc":
        if version == "190":
            assert "<PlatformToolset>v140</PlatformToolset>" in props
        elif version == "191":
            assert "<PlatformToolset>v141</PlatformToolset>" in props
    elif compiler == "clang":
        assert "<PlatformToolset>ClangCl</PlatformToolset>" in props
    if runtime == "dynamic":
        if runtime_type == "Release":
            assert "<RuntimeLibrary>MultiThreadedDLL</RuntimeLibrary>" in props
        else:
            assert "<RuntimeLibrary>MultiThreadedDebugDLL</RuntimeLibrary>" in props
    else:
        if runtime_type == "Release":
            assert "<RuntimeLibrary>MultiThreaded</RuntimeLibrary>" in props
        else:
            assert "<RuntimeLibrary>MultiThreadedDebug</RuntimeLibrary>" in props



import os
import textwrap

from conan.test.utils.tools import TestClient
from conan.internal.util.files import mkdir


class TestRemoveSubsetting:

    def test_remove_options(self):
        # https://github.com/conan-io/conan/issues/2327
        # https://github.com/conan-io/conan/issues/2781
        client = TestClient()
        conanfile = """from conan import ConanFile
class Pkg(ConanFile):
    options = {"opt1": [True, False], "opt2": [True, False]}
    default_options = {"opt1": True, "opt2": False}
    def config_options(self):
        del self.options.opt2
    def build(self):
        assert "opt2" not in self.options
        self.options.opt2
"""
        client.save({"conanfile.py": conanfile})
        build_folder = os.path.join(client.current_folder, "build")
        mkdir(build_folder)
        client.current_folder = build_folder
        client.run("install ..")
        client.run("build ..", assert_error=True)
        assert "ConanException: option 'opt2' doesn't exist" in client.out
        assert "Possible options are ['opt1']" in client.out

    def test_remove_setting(self):
        # https://github.com/conan-io/conan/issues/2327
        client = TestClient()
        conanfile = """from conan import ConanFile
class Pkg(ConanFile):
    settings = "os", "build_type"
    def configure(self):
        del self.settings.build_type

    def build(self):
        self.settings.build_type
"""
        client.save({"conanfile.py": conanfile})
        build_folder = os.path.join(client.current_folder, "build")
        mkdir(build_folder)
        client.current_folder = build_folder

        client.run("build ..", assert_error=True)
        assert "'settings.build_type' doesn't exist" in client.out


def test_settings_and_options_rm_safe():
    client = TestClient()
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    class Pkg(ConanFile):
        settings = "os", "build_type", "compiler"
        options = {"opt1": [True, False], "opt2": [True, False]}
        default_options = {"opt1": "True", "opt2": "False"}

        def configure(self):
            # setting
            self.settings.rm_safe("build_type")
            # sub-setting
            self.settings.rm_safe("compiler.version")
            # wrong settings
            self.settings.rm_safe("fake_field")
            self.settings.rm_safe("fake_field.version")

        def config_options(self):
            # option
            self.options.rm_safe("opt2")
            # wrong option
            self.options.rm_safe("opt15")

        def build(self):
            try:
                self.settings.build_type
            except Exception as exc:
                self.output.warning(str(exc))
            try:
                self.settings.compiler.version
            except Exception as exc:
                self.output.warning(str(exc))
            try:
                self.options.opt2
            except Exception as exc:
                self.output.warning(str(exc))
            assert "opt2" not in self.options
    """)
    client.save({"conanfile.py": conanfile})
    client.run("install .")
    client.run("build .")
    assert "'settings.build_type' doesn't exist" in client.out
    assert "'settings' possible configurations are ['compiler', 'os']" in client.out
    assert "'settings.compiler.version' doesn't exist" in client.out
    assert "'settings.compiler' possible configurations are [" in client.out
    assert "option 'opt2' doesn't exist" in client.out
    assert "Possible options are ['opt1']" in client.out



import json
import platform
import textwrap

import pytest

from conan.test.utils.mocks import ConanFileMock
from conan.tools.env.environment import environment_wrap_command
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


@pytest.fixture(scope="module")
def client():
    openssl = textwrap.dedent(r"""
        import os
        from conan import ConanFile
        from conan.tools.files import save, chdir
        class Pkg(ConanFile):
            settings = "os"
            package_type = "shared-library"
            def package(self):
                with chdir(self, self.package_folder):
                    echo = "@echo off\necho MYOPENSSL={}!!".format(self.settings.os)
                    save(self, "bin/myopenssl.bat", echo)
                    save(self, "bin/myopenssl.sh", echo)
                    os.chmod("bin/myopenssl.sh", 0o777)
            """)

    cmake = textwrap.dedent(r"""
        import os
        from conan import ConanFile
        from conan.tools.files import save, chdir
        class Pkg(ConanFile):
            settings = "os"
            requires = "openssl/1.0"
            def package(self):
                with chdir(self, self.package_folder):
                    echo = "@echo off\necho MYCMAKE={}!!".format(self.settings.os)
                    save(self, "mycmake.bat", echo + "\ncall myopenssl.bat")
                    save(self, "mycmake.sh", echo + "\n myopenssl.sh")
                    os.chmod("mycmake.sh", 0o777)

            def package_info(self):
                self.buildenv_info.append_path("PATH", self.package_folder)
            """)

    client = TestClient()
    client.save({"tool/conanfile.py": GenConanfile(),
                 "cmake/conanfile.py": cmake,
                 "openssl/conanfile.py": openssl})

    client.run("create tool --name=tool --version=1.0")
    client.run("create openssl --name=openssl --version=1.0")
    client.run("create cmake --name=mycmake --version=1.0")
    return client


@pytest.mark.parametrize("build_profile", ["", "-pr:b=default"])
def test_build_require_test_package(build_profile, client):
    test_cmake = textwrap.dedent(r"""
        import os, platform, sys
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os"

            def requirements(self):
                self.tool_requires(self.tested_reference_str)

            def build(self):
                mybuild_cmd = "mycmake.bat" if platform.system() == "Windows" else "mycmake.sh"
                self.run(mybuild_cmd)

            def test(self):
                pass
        """)

    # Test with extra build_requires to check it doesn't interfere or get deleted
    client.save({"cmake/test_package/conanfile.py": test_cmake})
    client.run("create cmake --name=mycmake --version=1.0 {} --build=missing".format(build_profile))

    def check(out):
        system = {"Darwin": "Macos"}.get(platform.system(), platform.system())
        assert "MYCMAKE={}!!".format(system) in out
        assert "MYOPENSSL={}!!".format(system) in out

    check(client.out)

    client.run("test cmake/test_package mycmake/1.0@ {}".format(build_profile))
    check(client.out)


def test_both_types(client):
    # When testing same package in both contexts, the 2 profiles approach must be used
    test_cmake = textwrap.dedent(r"""
        import os, platform
        from conan import ConanFile

        class Pkg(ConanFile):
            settings = "os"

            def requirements(self):
                self.requires(self.tested_reference_str)
                self.build_requires(self.tested_reference_str)

            def build(self):
                mybuild_cmd = "mycmake.bat" if platform.system() == "Windows" else "mycmake.sh"
                self.run(mybuild_cmd)

            def test(self):
                pass
        """)

    # Test with extra build_requires to check it doesn't interfere or get deleted
    client.save({"cmake/test_package/conanfile.py": test_cmake})
    # This must use the build-host contexts to have same dep in different contexts
    client.run("create cmake --name=mycmake --version=1.0 -pr:b=default --build=missing")

    def check(out):
        system = {"Darwin": "Macos"}.get(platform.system(), platform.system())
        assert "MYCMAKE={}!!".format(system) in out
        assert "MYOPENSSL={}!!".format(system) in out

    check(client.out)

    client.run("test cmake/test_package mycmake/1.0@ -pr:b=default")
    check(client.out)


def test_create_build_requires():
    # test that I can create a package passing the build and host context and package will get both
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os"

            def package_info(self):
                self.output.info("MYOS=%s!!!" % self.settings.os)
                self.output.info("MYTARGET={}!!!".format(self.settings_target.os))
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=br --version=0.1  --build-require -s:h os=Linux -s:b os=Windows")
    client.assert_listed_binary({"br/0.1": ("ebec3dc6d7f6b907b3ada0c3d3cdc83613a2b715", "Build")},
                                build=True)
    assert "br/0.1: MYOS=Windows!!!" in client.out
    assert "br/0.1: MYTARGET=Linux!!!" in client.out
    assert "br/0.1: MYOS=Linux!!!" not in client.out


def test_build_require_conanfile_text(client):
    client.save({"conanfile.txt": "[tool_requires]\nmycmake/1.0"}, clean_first=True)
    client.run("install . -g VirtualBuildEnv")
    ext = ".bat" if platform.system() == "Windows" else ".sh"
    cmd = environment_wrap_command(ConanFileMock(),"conanbuild", client.current_folder, f"mycmake{ext}")
    client.run_command(cmd)
    system = {"Darwin": "Macos"}.get(platform.system(), platform.system())
    assert "MYCMAKE={}!!".format(system) in client.out
    assert "MYOPENSSL={}!!".format(system) in client.out


def test_build_require_command_line_build_context(client):
    client.run("install --tool-requires=mycmake/1.0@ -g VirtualBuildEnv -pr:b=default")
    ext = ".bat" if platform.system() == "Windows" else ".sh"
    cmd = environment_wrap_command(ConanFileMock(),"conanbuild", client.current_folder, f"mycmake{ext}")
    client.run_command(cmd)
    system = {"Darwin": "Macos"}.get(platform.system(), platform.system())
    assert "MYCMAKE={}!!".format(system) in client.out
    assert "MYOPENSSL={}!!".format(system) in client.out


def test_install_multiple_tool_requires_cli():
    c = TestClient()
    c.save({"conanfile.py": GenConanfile()})
    c.run("create . --name=zlib --version=1.1")
    c.run("create . --name=cmake --version=0.1")
    c.run("create . --name=gcc --version=0.2")
    c.run("install --tool-requires=cmake/0.1 --tool-requires=gcc/0.2 --requires=zlib/1.1")
    c.assert_listed_require({"cmake/0.1": "Cache", "gcc/0.2": "Cache"}, build=True)
    c.assert_listed_require({"zlib/1.1": "Cache"})


def test_bootstrap_other_architecture():
    """ this is the case of libraries as ICU, that needs itself for cross-compiling
    """
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.build import cross_building

        class Pkg(ConanFile):
            name = "tool"
            version = "1.0"
            settings = "os"
            def build_requirements(self):
                if cross_building(self):
                    self.tool_requires("tool/1.0")
        """)
    c.save({"conanfile.py": conanfile})
    win_pkg_id = "ebec3dc6d7f6b907b3ada0c3d3cdc83613a2b715"
    linux_pkg_id = "9a4eb3c8701508aa9458b1a73d0633783ecc2270"

    c.run("create . -s:b os=Windows -s:h os=Windows")
    c.assert_listed_binary({"tool/1.0": (win_pkg_id, "Build")})
    assert "Build requirements" not in c.out

    # This is smart and knows how to build only the missing one "host" but not "build"
    c.run("create . -s:b os=Windows -s:h os=Linux --build=missing:tool*")
    c.assert_listed_binary({"tool/1.0": (linux_pkg_id, "Build")})
    c.assert_listed_binary({"tool/1.0": (win_pkg_id, "Cache")}, build=True)

    # This will rebuild all
    c.run("create . -s:b os=Windows -s:h os=Linux")
    c.assert_listed_binary({"tool/1.0": (linux_pkg_id, "Build")})
    c.assert_listed_binary({"tool/1.0": (win_pkg_id, "Build")}, build=True)

    c.run("graph build-order --requires=tool/1.0 -s:b os=Windows -s:h os=Linux --build=* "
          "--format=json", redirect_stdout="o.json")
    order = json.loads(c.load("o.json"))
    package1 = order[0][0]["packages"][0][0]
    package2 = order[0][0]["packages"][1][0]
    assert package1["package_id"] == win_pkg_id
    assert package1["depends"] == []
    assert package2["package_id"] == linux_pkg_id
    assert package2["depends"] == [win_pkg_id]


def test_bootstrap_cc():
    # https://github.com/conan-io/conan/issues/16758
    cc = textwrap.dedent("""
        from conan import ConanFile

        class CcConanfile(ConanFile):
            name = "cc"
            version = "1.0"
            options = {"bootstrap": [True, False]}
            default_options = {"bootstrap": True}

            def build_requirements(self):
                if self.options.bootstrap:
                    self.tool_requires("cc/1.0", options={"*:bootstrap": False})
        """)
    foo = GenConanfile("foo", "1.0").with_tool_requirement("cc/1.0")
    c = TestClient()
    c.save({"cc/conanfile.py": cc,
            "foo/conanfile.py": foo})
    c.run("create cc --build-require")
    # Both are build-requires
    c.assert_listed_binary({"cc/1.0": ("826727aac60b5956d1df5121a3921f26a6984f15", "Build")},
                           build=True)
    c.assert_listed_binary({"cc/1.0": ("96ae1d965ce6f2e2256ac0cfc3a35dcd4860c389", "Build")},
                           build=True)
    # This works fine, being able to build the 2 different binaries both bootstrapped and not
    c.run("list cc/1.0:*")
    assert "bootstrap: False" in c.out
    assert "bootstrap: True" in c.out
    c.run("create foo")
    c.assert_listed_binary({"cc/1.0": ("96ae1d965ce6f2e2256ac0cfc3a35dcd4860c389", "Cache")},
                           build=True)



import os
import sys
import textwrap

import pytest

from conan.internal.loader import ConanFileLoader, ConanFileTextLoader, load_python_file
from conan.errors import ConanException
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import save, chdir


class TestConanLoaderTxt:
    def test_conanfile_txt_errors(self):
        # Invalid content
        file_content = '''[requires}
OpenCV/2.4.10@phil/stable # My requirement for CV
'''
        with pytest.raises(ConanException, match="Bad syntax"):
            ConanFileTextLoader(file_content)

        file_content = '{hello}'
        with pytest.raises(ConanException, match="Unexpected line"):
            ConanFileTextLoader(file_content)

    def test_plain_text_parser(self):
        # Valid content
        file_content = '''[requires]
OpenCV/2.4.10@phil/stable # My requirement for CV
OpenCV2/2.4.10@phil/stable #
OpenCV3/2.4.10@phil/stable
[generators]
one # My generator for this
two
[options]
OpenCV:use_python=True # Some option
OpenCV:other_option=False
OpenCV2:use_python2=1
OpenCV2:other_option=Cosa #
'''
        parser = ConanFileTextLoader(file_content)
        exp = ['OpenCV/2.4.10@phil/stable',
               'OpenCV2/2.4.10@phil/stable',
               'OpenCV3/2.4.10@phil/stable']
        assert parser.requirements == exp

    def test_revision_parsing(self):
        # Valid content
        file_content = '''[requires]
OpenCV/2.4.10@user/stable#RREV1 # My requirement for CV
'''
        parser = ConanFileTextLoader(file_content)
        exp = ['OpenCV/2.4.10@user/stable#RREV1']
        assert parser.requirements == exp

    def test_load_conan_txt(self):
        file_content = '''[requires]
OpenCV/2.4.10@phil/stable
OpenCV2/2.4.10@phil/stable
[tool_requires]
Mypkg/1.0.0@phil/stable
[generators]
one
two
[options]
OpenCV/*:use_python=True
OpenCV/*:other_option=False
OpenCV2/*:use_python2=1
OpenCV2/*:other_option=Cosa
'''
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "file.txt")
        save(file_path, file_content)
        loader = ConanFileLoader(None, None)
        ret = loader.load_conanfile_txt(file_path)

        assert len(ret.requires.values()) == 3
        assert ret.generators == ["one", "two"]
        assert ret.options.dumps() == 'OpenCV/*:other_option=False\n' \
                                              'OpenCV/*:use_python=True\n' \
                                              'OpenCV2/*:other_option=Cosa\n' \
                                              'OpenCV2/*:use_python2=1'

    def test_load_options_error(self):
        conanfile_txt = textwrap.dedent("""
            [options]
            myoption: myvalue
        """)
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "file.txt")
        save(file_path, conanfile_txt)
        loader = ConanFileLoader(None, None)
        with pytest.raises(ConanException,
                                    match=r"Error while parsing \[options\] in conanfile.txt\n" \
                                    r"Options should be specified as 'pkg/\*:option=value'"):
            loader.load_conanfile_txt(file_path)

    def test_layout_not_predefined(self):
        txt = textwrap.dedent("""
                    [layout]
                    missing
                """)
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "conanfile.txt")
        save(file_path, txt)
        with pytest.raises(ConanException) as exc:
            loader = ConanFileLoader(None, None)
            loader.load_conanfile_txt(file_path)
        assert "Unknown predefined layout 'missing'" in str(exc.value)

    def test_layout_multiple(self):
        txt = textwrap.dedent("""
                    [layout]
                    cmake_layout
                    vs_layout
                """)
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "conanfile.txt")
        save(file_path, txt)
        with pytest.raises(ConanException) as exc:
            loader = ConanFileLoader(None, None)
            loader.load_conanfile_txt(file_path)
        assert "Only one layout can be declared in the [layout] section of the conanfile.txt" \
               in str(exc.value)


class TestImportModuleLoader:
    @staticmethod
    def _create_and_load(myfunc, value, subdir_name, add_subdir_init):
        subdir_content = textwrap.dedent("""
            def get_value():
                return {value}
            def {myfunc}():
                return "{myfunc}"
        """)

        side_content = textwrap.dedent("""
            def get_side_value():
                return {value}
            def side_{myfunc}():
                return "{myfunc}"
        """)

        conanfile = textwrap.dedent("""
            import pickle
            from {subdir}.api import get_value, {myfunc}
            from file import get_side_value, side_{myfunc}
            from fractions import Fraction
            def conanfile_func():
                return get_value(), {myfunc}(), get_side_value(), side_{myfunc}(), str(Fraction(1,1))
        """)
        expected_return = (value, myfunc, value, myfunc, "1")

        tmp = temp_folder()
        with chdir(tmp):
            save("conanfile.py", conanfile.format(value=value, myfunc=myfunc, subdir=subdir_name))
            save("file.py", side_content.format(value=value, myfunc=myfunc))
            save("{}/api.py".format(subdir_name), subdir_content.format(value=value, myfunc=myfunc))
            if add_subdir_init:
                save("__init__.py", "")
                save("{}/__init__.py".format(subdir_name), "")

        loaded, module_id = load_python_file(os.path.join(tmp, "conanfile.py"))
        return loaded, module_id, expected_return

    @pytest.mark.parametrize("sub1,sub2", [(True, False), (False, True), (False, False)])
    def test_py3_recipe_colliding_init_filenames(self, sub1, sub2):
        myfunc1, value1 = "recipe1", 42
        myfunc2, value2 = "recipe2", 23
        loaded1, module_id1, exp_ret1 = self._create_and_load(myfunc1, value1, "subdir", sub1)
        loaded2, module_id2, exp_ret2 = self._create_and_load(myfunc2, value2, "subdir", sub2)

        assert module_id1 != module_id2
        assert loaded1.conanfile_func() == exp_ret1
        assert loaded2.conanfile_func() == exp_ret2

    def test_recipe_colliding_filenames(self):
        myfunc1, value1 = "recipe1", 42
        myfunc2, value2 = "recipe2", 23
        loaded1, module_id1, exp_ret1 = self._create_and_load(myfunc1, value1, "subdir", True)
        loaded2, module_id2, exp_ret2 = self._create_and_load(myfunc2, value2, "subdir", True)

        assert module_id1 != module_id2
        assert loaded1.conanfile_func() == exp_ret1
        assert loaded2.conanfile_func() == exp_ret2

    @pytest.mark.parametrize("add_subdir_init", [(True, ), (False, )])
    def test_wrong_imports(self, add_subdir_init):
        myfunc1, value1 = "recipe1", 42

        # Item imported does not exist, but file exists
        # We use some existing and imported Python stdlib import
        with pytest.raises(ConanException, match="Unable to load conanfile in"):
            self._create_and_load(myfunc1, value1, "textwrap", add_subdir_init)

        # File does not exists in already existing module
        with pytest.raises(ConanException, match="Unable to load conanfile in"):
            self._create_and_load(myfunc1, value1, "conans", add_subdir_init)

    def test_helpers_python_library(self):
        mylogger = """
value = ""
def append(data):
    global value
    value += data
"""
        temp = temp_folder()
        save(os.path.join(temp, "myconanlogger.py"), mylogger)

        conanfile = "import myconanlogger"
        temp1 = temp_folder()
        save(os.path.join(temp1, "conanfile.py"), conanfile)
        temp2 = temp_folder()
        save(os.path.join(temp2, "conanfile.py"), conanfile)

        try:
            sys.path.append(temp)
            loaded1, _ = load_python_file(os.path.join(temp1, "conanfile.py"))
            loaded2, _ = load_python_file(os.path.join(temp2, "conanfile.py"))
            assert loaded1.myconanlogger == loaded2.myconanlogger
            assert loaded1.myconanlogger.value == loaded2.myconanlogger.value
        finally:
            sys.path.remove(temp)



import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.fixture
def client():
    conanfile = textwrap.dedent("""
       from conan import ConanFile
       class Pkg(ConanFile):
           generators = "VirtualRunEnv"
           def generate(self):
               for var in (1, 2):
                   v = self.runenv.vars(self).get("MyVar{}".format(var))
                   self.output.info("MyVar{}={}!!".format(var, v))
       """)
    profile1 = textwrap.dedent("""
      [runenv]
      MyVar1=MyValue1_1
      MyVar2=MyValue2_1
      """)
    client = TestClient()
    client.save({"conanfile.py": conanfile,
                 "profile1": profile1})
    return client


def test_buildenv_profile_cli(client):
    client.run("install . -pr=profile1")
    assert "conanfile.py: MyVar1=MyValue1_1!!" in client.out
    assert "conanfile.py: MyVar2=MyValue2_1!!" in client.out
    env = client.load("conanrunenv.sh")
    assert "MyValue1_1" in env
    assert "MyValue2_1" in env
